# ###############################
#
# NetRS.py
#
# Code made by fjalar@vedur.is
# Iceland Met Office
# 2011-2016
#
# ###############################

######## IMPORT LIBRARIES ########
import sCurl, re, sys
import checkPort as checkport


######## DEFINE CLASS ########
class NetRS:

    # ALL TEXT HERE IN THE DOC NEEDS TO BE REVISED LATER ON
    """A Receiver class designed for Trimble NetRS receiver type.

    Public functions:
    getConnection       -- Get the status of the connection to the router and receiver.
    getVolt             -- Get voltage as measured from one of two ports on the receiver.
    getTemp             -- Get the temperature as measured in the containment of the receiver.
    getLogging          -- Get the logging status of the receiver.
    getSessions         -- List all sessions defined on the receiver and their status; active or inactive.
    getDirList          -- Get a list of subdirectories for a certain directiory.
    getFileList         -- Get a list of files for a specific directory passed as an argument.
    getPosition         -- Get the current position of the receiver.
    getTracking         -- Get the current tracking of satellites.
    getFirmwareVersion  -- Get the current firmware version for the receier.
    getStatus           -- A collective report for the receiver; volt, temp, logging and sessions.
    info                -- Reports information on the receiver object.

    Private functions:
    __getStuff          -- The function that combines the similar actions of getTemp, getVolt, getLogging and getSessions.
    __parceFancy        -- Parcing method used by the Public methods to trim and cut the http result and return values in dictionaries instead of a singel string.
    """

    def __init__(self,station_info):


        # 1 # Parameters saved and type set to string
        self.station_info = station_info
        self.sid = self.station_info['station']['id']
        self.userpass = None
        self.CONNECTION_STATUS = {'router':None,'receiver':None}

        # 2 # The router IP here is the ip on the router/receiver connected to a SIL station. IP is on the form 192.168.100.XX
        if station_info['proxy']['httpport'] != '':

            self.ip = 'localhost'
            self.ipport = self.ip
            self.port = station_info['proxy']['httpport']


        elif self.station_info['proxy']['httpport'] == '':

            self.ip = station_info['router']['ip']
            self.ipport = self.ip
            self.port = station_info['receiver']['httpport']

        # 4 # If username and password is required to access the receiver, the user/pass info will be passed to the CURL command in the
            # self.userpass variable (dicitonary)
        if station_info['receiver']['user'] and station_info['receiver']['pwd']:
            self.userpass = {'user':station_info['receiver']['user'],'pass':station_info['receiver']['pwd']}

        # 5 # Set the default value for DEBUG
        self.DEBUG = False

        # 6 # Expand IP with port if port is defined
        if self.port:
            self.ipport = '%s:%s'% (self.ip,self.port)

        # 7 # Instantiate gpsCURL object for URL stuff
        self.curl = sCurl.sCurl()

        # 8 # Check if station is alive
        self.CONNECTION_STATUS = self.getConnection()

    def getConnection(self):
        """Fetches connection status with the router and receiver"""

        result ={'router':False,'receiver':False,}

        result = checkport.isOpen(self.ip,self.port,self.DEBUG)

        return result

    def getVolt(self):
        """Fetches voltage information from receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = {'value':None, 'error':None}

        # 2 # Get result for different voltport
        self.voltport = '1'
        resp_vp1 = self.__getStuff('volt')
        self.voltport = '2'
        resp_vp2 = self.__getStuff('volt')

        # Append resulting dicts in a proper format for return.. looks messy :/
        # Only the actual voltage value is returned and not the both ports.
        # At least not for now...
        if self.DEBUG: print resp_vp1,resp_vp2

        # 3 # Fish the proper voltage and add to result
        if resp_vp1 ['value'] != None and resp_vp2['value'] != None:
            if float(resp_vp1['value']) > float(resp_vp2['value']):
                if self.DEBUG: print "%s port 1: %s bigger than port 2:%s " % (currFunc,resp_vp1['value'],resp_vp2['value'])
                result['value'] = resp_vp1['value']
            else:
                if self.DEBUG: print "%s port 1: %s less than port 2:%s " % (currFunc,resp_vp1['value'],resp_vp2['value'])
                result['value'] = resp_vp2['value']
            result['error'] = None

        if self.DEBUG: print "%s The return is: %s" % (currFunc, result)
        # 4 # Return
        return result

    def getTemp(self):
        """Fetches temperature information from receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = self.__getStuff('temp')
        return result

    def getLogging(self):
        """Fetches information about the logging status of the receiver; What files are currently being created"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = self.__getStuff('logging')
        return result

    def getSessions(self):
        """Fetches information about defined sessions from the receiver; which sessions are active and which are not."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = self.__getStuff('sessions')
        return result

    def getFileList(self,months=99):
        """Fetches file listing from the receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        # 1 # Define variables.. dictionary and a list
        filedict = {'month':'','freq':'','file':'','size':''}   # Dictionary for a single file
        fileList = []                                           # List of file dictionaries
        rootDir = ''                                            # The root of the directory list on this type of receiver
        monthCount = 0                                          # Counter variable for months
        monthFreq = {'value':None,'error':None}
        result = {'value':None,'error':None}

        # 2 # Get list of month directories from the root
        monthList = self.getDirList(rootDir)

        # 3 # Always the lesser value is chosen. This is done to prevent index-out-of-bound problems
        months = min(len(monthList['value']),months)

        # 4 # Traverse the whole tree and map
        for month in reversed(monthList['value']):
            # A simple counter that breaks if nubmer of months passed as value is reached.
            # 4.1 # If no value was passed months is set to 99 which basically means that the whole tree will be traversed.
            if monthCount == months:
                break
            monthCount += 1

            if self.DEBUG: print month['name']

            # 4.2 # Get list of frequency directories form the month
            monthFreq = self.getDirList(rootDir+month['name'])
	    if monthFreq:
		    if self.DEBUG: print monthFreq

		    # 4.3 # Loop through each frequency directory
		    for freq in monthFreq['value']:

		        # 4.3.1 # Get list of each file in the directory
		        files = self.getDirList(rootDir+month['name']+'/'+freq['name'])

		        # 4.3.2 # Loop through each file
		        for nfile in files['value']:

		            # 4.3.2.1 # Create a new dictionary for each fiel from all the collected data..
		            filedict = {'month':month['name'],'freq':freq['name'],'file':nfile['name'],'size':nfile['size']}

		            # 4.3.2.2 # Add the new file dictionary to a file list
		            fileList.append(filedict)

		    result['value'] = fileList

	    else:

		result['error'] = nothFreq['error']


        # 6 # Return
        return result

    def getDirList(self,directory=''):
        """Fetches directory listing from the receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = self.__getStuff('dirlist',directory)
        return result

    def getPosition(self):
        """Fetches the set position of the receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = self.__getStuff('position')
        return result

    def getTracking(self):
        """Fetches the set position of the receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = self.__getStuff('tracking')
        return result

    def getFirmwareVersion(self):
        """Fetches the firmware version for the receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        result = self.__getStuff('firmwareversion')
        return result

    def getStatus(self):
        """Compiles a status report for the receiver; temp, volt, logging, session and returns as a dictionary """

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        # 1 # Define dictionary
        result = {'connection':self.CONNECTION_STATUS,'temp':{'value':None},'volt':{'value':None},'logging':{'value':None},'session':{'value':None},'position':{'value':None}}

        # 2 # Do the funky dance..
        if self.CONNECTION_STATUS['router']:
            result['connection'] = self.getConnection()
            result['temp'] = self.getTemp()
            result['volt'] = self.getVolt()
            result['logging'] = self.getLogging()
            result['session'] = self.getSessions()
            result['position'] = self.getPosition()
            #result['tracking'] = self.getTracking()

        # 3 # Return
        return result

    def test(self):
        """Simple method that calls all the public methods in the class without any parameters"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        test = {'connection':'','temp':'','volt':'','logging':'','session':'','filelist':'','position':''}

        #self.setDebug()
        test['connection'] = self.getConnection()
        test['temp'] = self.getTemp()
        test['volt'] = self.getVolt()
        test['logging'] = self.getLogging()
        test['sessions'] = self.getSessions()
        test['filelist'] = self.getFileList()
        test['position'] = self.getPosition()

        return test

    def setDebug(self):
        """Debug ON/OFF switch."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        if self.DEBUG:
            self.DEBUG = False
            print '%s Debug is now off OFF' % currFunc
        else:
            self.DEBUG = True
            print '%s Debug is now ON' % currFunc

    def info(self):
        """Information on the object's attributes"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        print ''
        print '%s SID = %s' % (currfunc,self.sid)
        print '%s IP = %s' % (currfunc,self.ip)
        print '%s PORT = %s' % (currfunc,self.port)
        print '%s DEBUG = %s' % (currfunc,self.DEBUG)
        print '%s ATTRIBUTES =  %s' % (currfunc,dir(self))

    def __getStuff(self, get_type, recv_dir='',filename=''):
        """Method that basically is a big switch statement and is intended to minimize copy-paste in the logical code."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        #if self.DEBUG: print 'DEBUG is ON'
        # 1 # Define variables
        result = {'value':None,'error':None}

        #if self.DEBUG: print '%s Station info:' % (currFunc)
        #if self.DEBUG: print "%s Station INFO: %s" % (currFunc,self.station_info)

        # 2 # Define command based on get_type
        if get_type == 'volt':
            url_string = 'http://%s/prog/show?Voltage&input=%s' % (self.ipport,self.voltport)
        if get_type =='temp':
            url_string = 'http://%s/prog/show?Temperature' % (self.ipport)
        if get_type =='logging':
            url_string = 'http://%s/prog/show?loggingstatus' % (self.ipport)
        if get_type =='sessions':
            url_string = 'http://%s/prog/show?sessions' % (self.ipport)
        if get_type =='dirlist':
            url_string = 'http://%s/prog/show?loggedfiles&directory=/%s' % (self.ipport,recv_dir)
        if get_type =='position':
            url_string = 'http://%s/prog/show?position' % (self.ipport)
        if get_type =='tracking':
            url_string = 'http://%s/prog/show?trackingstatus' % (self.ipport)
        if get_type == 'firmwareversion':
            url_string = 'http://%s/prog/show?firmwareversion' % (self.ipport)


        if self.DEBUG: print '%s URL string: %s' % (currFunc,url_string)
        if self.DEBUG: print "%s Router  status: %s " % (currFunc,self.CONNECTION_STATUS['router'])
        if self.DEBUG: print "%s Receiver status: %s" % (currFunc,self.CONNECTION_STATUS['receiver'])

        if self.CONNECTION_STATUS['router'] == True and self.CONNECTION_STATUS['receiver'] == True:
            if self.DEBUG:
                print "%s Both receiver (stat:%s) and router (stat:%s) are alive.. now some action!" % (currFunc,self.CONNECTION_STATUS['router'],self.CONNECTION_STATUS['receiver'])

            # 3 # Some CURL magic
            if url_string != '':
                if self.DEBUG: print "%s This is the URL string: %s " % (currFunc,url_string)
                curl_result = self.curl.getURL(url_string,self.userpass,self.DEBUG)
            else:
                if self.DEBUG: print "No URL string defined..."

	    # If the was no error in the reply from the sCurl then continue
	    if curl_result['value']:

		    # 4 # Some parsing magic
		    if self.DEBUG: print '%s Before parsing, value: %s error: %s ' % (currFunc,curl_result['value'],curl_result['error'])
		    result['value'] = self.__parseFancy(get_type,curl_result['value'])
		    if self.DEBUG: print '%s After parsing, value: %s error: %s ' % (currFunc,result['value'],result['error'])
	    else:
		result['error'] = "Connection ERROR: "+curl_result['error']

        else:
            result['value'] = None

            if not self.CONNECTION_STATUS['router']:
                result['error'] = "Connection ERROR: ROUTER OFFLINE"
            else:
                result['error'] = "Connection ERROR: RECEIVER UNREACHABLE"

        # 5 # Return
        return result

    def __parseFancy(self, string_type, string):
        """Parses the return strings from the receiver and retuns only the relevant information in a dictionary """

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        # 1 # Define variables
        result = None

        if self.DEBUG: print '%s String type: %s' % (currFunc,string_type)

        # 2 # If string type is 'volt'
        if string_type == 'volt':

            # 2.1 # Define variables
            patt_found = None

            if self.DEBUG: print "%s String before parsing: %s" % (currFunc,string)
            if self.DEBUG: print "%s Voltport: %s" % (currFunc,self.voltport)

            # 2.2 # Define regexp pattern
            regex=re.compile('(?:input|port)=%s volts=(\d+(?:\.\d*)?)' % self.voltport)

            try:
                patt_found = regex.search(string)
                result = patt_found.group(1)

            except:
                if self.DEBUG: print '%s ERROR: %s failed' % (currFunc,string_type)

        # 3 # If string type is 'temp'
        if string_type == 'temp':

            # 3.1 # Define variables
            patt_found = None

            # 3.2 # Define regexp pattern
            regex=re.compile('Temperature temp=(\d+(?:\.\d*)?)')

            try:
                patt_found = regex.search(string)
                result = patt_found.group(1)
            except:
                if self.DEBUG: print '%s ERROR: %s failed' % (currFunc,string_type)

        # 4 # If string type is 'logging' or 'sessions'
        if string_type == 'logging' or string_type == 'sessions':

            # 4.1 # Define variables
            result=[]
            patt_found = None

            # 4.2 # Define regexp pattern
            regex=re.compile("\s*(\S+)=(\S+)\s*")

            try:
                for line in string.split('\n'):
                    if 'name' in line:
                        if self.DEBUG: print 'Line: ', line
                        patt_found=regex.findall(line)
                        if self.DEBUG: print '%s Pattern: %s' % (currFunc,patt_found)
                        result.append(dict(patt_found))

            except:
                if self.DEBUG: print 'ERROR: %s failed' % string_type

        # 5 # If string type is 'filelist'
        if string_type == 'dirlist':

            # 5.1 # Define variables
            result=[]
            patt_found = None

            # 5.2 # Define regexp pattern
            regex=re.compile("\s*(\S+)=(\S+)\s*")

            try:
                for line in string.split('\n'):
                    if 'name' in line:
                        if self.DEBUG: print 'Line: ', line
                        patt_found=regex.findall(line)
                        if self.DEBUG: print '%s Pattern: %s' % (currFunc,patt_found)
                        result.append(dict(patt_found))

            except:
                if self.DEBUG: print '%s ERROR: %s failed' % (currFunc,string_type)

        # 6 # If string type is 'filelist'
        if string_type == 'position':

            # 6.1 # Define variables
            result = {'latitude':None,'longitude':None,'altitude':None}
            patt_found = None

            # 6.2 # Define regexp pattern
            regex=re.compile("(\w+)\s*(-?\w+.\w+)\s+(\w{3,6})")


            try:
                for line in string.split('\n'):

                    if line.startswith('Latitude'):
                        if self.DEBUG: print 'Line: ', line
                        patt_found=regex.findall(line)
                        if self.DEBUG: print '%s Pattern: %s' % (currFunc,patt_found)
                        result['latitude'] = patt_found[0][1]

                    if line.startswith('Longitude'):
                        if self.DEBUG: print 'Line: ', line
                        patt_found=regex.findall(line)
                        if self.DEBUG: print '%s Pattern: %s' % (currFunc,patt_found)
                        result['longitude'] = patt_found[0][1]

                    if line.startswith('Altitude'):
                        if self.DEBUG: print 'Line: ', line
                        patt_found=regex.findall(line)
                        if self.DEBUG: print '%s Pattern: %s' % (currFunc,patt_found)
                        result['altitude'] = patt_found[0][1]

            except:
                if self.DEBUG: print '%s ERROR: %s failed' % (currFunc,string_type)

        # 7 # If string type is 'tracking'
        if string_type == 'tracking':

            # 7.1 # Define variables
            result=[]
            patt_found = None

            # 7.2 # Define regexp pattern
            regex=re.compile("\s*(\S+)=(\S+)\s*")

            try:
                for line in string.split('\n'):
                    if line.startswith('Chan'):
                        if self.DEBUG: print 'Line: ', line
                        patt_found=regex.findall(line)
                        if self.DEBUG: print '%s Pattern: %s' % (currFunc,patt_found)
                        result.append(dict(patt_found))
            except:
                if self.DEBUG: print '%s ERROR: %s failed' % (currFunc, string_type)

        # 8 # If string type is 'firmwareversion'
        if string_type == 'firmwareversion':

            # 8.1 # Define variables
            patt_found = None

            # 8.2 # Define regexp pattern
            regex=re.compile("\s*(\S+)=(\S+)\s*")

            try:
                patt_found=regex.findall(string)
                result = dict(patt_found)

            except:
                if self.DEBUG: print '%s ERROR: %s failed' % (currFunc,string_type)

        # 9 # Return
        return result

# HERE IS THE END!


from setuptools import setup, find_packages

setup(name='geofunc',
      version='0.1',
      description='Time and date modules capable of handling GPS time',
      url='http://github.com/imo/geofunc',
      author='Benedikt G. Ofeigsson',
      author_email='bgo@vedur.is',
      license='Icelandic Met Office',
      package_dir={'geofunc': 'geofunc'},
      packages=find_packages(),
      install_requires=['pyproj','scipy'],
      zip_safe=False)



from setuptools import setup

setup(name='gpsUtils',
      version='0.1',
      description='Utils for the GPS modules',
      url='http://github.com/imo/gps/utils',
      author='Benedikt G. Ofeigsson',
      author_email='bgo@vedur.is',
      license='Icelandic Met Office',
      py_modules=['gpsUtils'],
      )



# ###############################
#
# Leica.py 
#
# Code made by fjalar@vedur.is 
# Iceland Met Office
# 2014
#
# ###############################

######## IMPORT LIBRARIES ########

import sCurl, re, sys, datetime
import checkPort as checkport
    
######## DEFINE CLASS ########

class Leica:

    # ALL TEXT HERE IN THE DOC NEEDS TO BE REVISED LATER ON
    """A Receiver class designed for Leica receiver type.
    
    Public functions:
    getConnection       -- Get the status of the connection to the router and receiver.
    getVolt             -- Not implemented.
    getTemp             -- Not implemented.
    getLogging          -- Not implemented.
    getSessions         -- Not implemented.
    getDirList          -- Not implemented.
    getFileList         -- Not implemented.
    getPosition         -- Not implemented.
    getTracking         -- Not implemented.
    getFirmwareVersion  -- Not implemented.
    getStatus           -- A collective report for the receiver; logging and session. 
    info                -- Reports information on the receiver object.

    Private functions:
    __getStuff          -- Not present.
    __parceFancy        -- Not present.
    """

    def __init__(self,station_info):
        
        # 1 # Parameters saved and type set to string
        self.station_info = station_info        
        self.sid = self.station_info['station']['id']  
        self.userpass = None
        self.CONNECTION_STATUS = {'router':None,'receiver':None}
        
        # 2 # The router IP here is the ip on the router/receiver connected to a SIL station. IP is on the form 192.168.100.XX
        if self.station_info['proxy']['sshserver'] == '':

            self.ip = station_info['router']['ip']
            self.ipport = self.ip
            self.port = station_info['receiver']['httpport']

        # 3 # If the connection uses a proxy through kjarni the IP is set as localhost to use the ssh-tunnel from the localhost (rek/rek2)
        if station_info['proxy']['sshserver'] != '':         

            self.ip = 'localhost'
            self.ipport = self.ip
            self.port = station_info['proxy']['httpport']

        # 4 # If username and password is required to access the receiver, the user/pass info will be passed to the CURL command in the
            # self.userpass variable (dicitonary)
        if station_info['receiver']['user'] and station_info['receiver']['pwd']:
            self.userpass = {'user':station_info['receiver']['user'],'pass':station_info['receiver']['pwd']}

        # 5 # Set the default value for DEBUG
        self.DEBUG = False
        
        # 6 # Expand IP with port if port is defined
        if self.port:
            self.ipport = '%s:%s'% (self.ip,self.port)
 
        # 8 # Instantiate sCurl object for URL stuff
        self.curl = sCurl.sCurl()
       
        # 8 # Check if station is alive
        self.CONNECTION_STATUS = self.getConnection()
    ## ------------------- getConnection() -------------------##

    def getConnection(self):
        """Fetches connection status with the router and receiver"""
        
        result ={'router':None,'receiver':None,}

        result = checkport.isOpen(self.ip,self.port,self.DEBUG)

        return result


    ## ------------------- getVolt() -------------------##

    def getVolt(self):
        """Fetches voltage information from receiver"""
        result = {'value':'N/S','error':''}
        return result 

    ## ------------------- getTemp() -------------------##

    def getTemp(self):
        """Fetches temperature information from receiver"""
        result = {'value':'N/S','error':''}
        return result 

    ## ------------------- getLogging() -------------------##
    def getLogging(self):
        """Fetches information about the logging status of the receiver; What files are currently being created"""        

        result = {'value':'N/S','error':''}
        return result 

    ## ------------------- getSession() -------------------##

    def getSessions(self):
        """Fetches information about defined sessions from the receiver; which sessions are active and which are not."""

        result = {'value':'N/S','error':''}
        return result 

    def getPosition(self):
        """Fetches information about the position of the receiver.. from the receiver."""
        
        result = {'value':{'latitude':'N/S','longitude':'N/S','altitude':'N/S'},'error':''}

        return result 

    def getFileList(self,directory=''):
        """Fetches directory listing from a receiver"""

        result = {'value':'N/S','error':''}
        return result 

    def getTracking(self):
        """Fetches tracking status for a receiver"""

        result = {'value':'N/S','error':''}
        return result 

    def getFirmwareVersion(self):
        """Fetches the firmware version for the receiver"""

        result = {'value':'N/S','error':''}
        return result 

    def getStatus(self):
        """Compiles a status report for the receiver; temp, volt, logging, session and returns as a dictionary """
        
        # 1 # Define local variables 

        report = {'connection':'','temp':'','volt':'','logging':'','session':'','position':''}
        
        # 2 # Do the funky dance..
        report['connection'] = self.getConnection()
        report['temp'] = self.getTemp()
        report['volt'] = self.getVolt()
        report['logging'] = self.getLogging()
        report['session'] = self.getSessions()
        report['position'] = self.getPosition()  
  
        
        return report
        
    def setDebug(self):
        """Debug ON/OFF switch."""

        if self.DEBUG:
            self.DEBUG = False
            print 'Debug OFF'
        else:
            self.DEBUG = True
            print 'Debug ON'
        

    def info(self):
        """Information on the object's attributes"""
        print ''
        print 'SID = %s' % self.sid
        print 'IP = %s' % self.ip
        print 'PORT = %s' % self.port
        print 'DEBUG = %s' % self.DEBUG
        print 'ATTRIBUTES =  %s' % dir(self)


    def test(self):
        """Simple method that calls all the public methods in the class without any parameters"""

        test = {'connection':'','temp':'','volt':'','logging':'','session':'','filelist':'','position':''}        

        #self.setDebug()
        test['connection'] = self.getConnection()
        test['temp'] = self.getTemp()
        test['volt'] = self.getVolt()
        test['logging'] = self.getLogging()
        test['sessions'] = self.getSessions()
        test['filelist'] = self.getFileList()
        test['position'] = self.getPosition()

        return test

  
       # ATT!!     
        # The rest is dismissed but a good template for both _getStuff and _parseFancy can be found in the other receiver moudles.

# HERE IS THE END!



from setuptools import setup, find_packages

setup(name='pconnect',
      version='1.2',
      description='Connect establishes connections via proxys',
      url='http://repo.vedur.is:18080/svn/GPS/library/pconnect/branches/1.0/',
      author='Fjalar Sigurdarson',
      author_email='fjalar@vedur.is',
      packages = find_packages(),
      scripts=['proxytunnel'],
      install_reqires=['re','getopt'],
      )



# ###############################
#
# dblib.py Database Function Library
#
# Code made by fjalar@vedur.is 
# Iceland Met Office
# 2014
#
# ###############################


######## IMPORT LIBRARIES ########

import sys
# mysql library
#import MySQLdb as mdb
# postgresql library
#import _pg
import psycopg2 as pgdb


class dblib:

    # ALL TEXT HERE IN THE DOC NEEDS TO BE REVISED LATER ON
    """A class designed to ....
    
    Public functions:
      getNextBatch()                --
      addReportToDB()               --
      setDebug()                    --

    Private functions:
      No private functions defined.
    """

    def __init__(self,db_info):
        
        # 1 # Predefine few global variables for the class
        self.DEBUG      = False
        self.HOST       = db_info['host']
        self.DBNAME     = db_info['dbname']
        self.USER       = db_info['user']
        self.PASSWORD   = db_info['password']
        self.DB_TABLE   = db_info['dbtable']

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        self.CONNECT_STRING = 'host=%s dbname=%s user=%s password=%s' % (self.HOST,self.DBNAME,self.USER,self.PASSWORD)


    # ------------------ setDebug() ------------------ #

    def setDebug(self):
        """Debug ON/OFF switch - handy when debugging."""
        if self.DEBUG:
            self.DEBUG = False
            print 'Debug OFF'
        else:
            self.DEBUG = True
            print 'Debug ON'    

    # ------------------ getNextBatch() ------------------ #

    def getNextBatch(self):
        """Fetches the next batch number in the checkcomm table from the database to be incremented by 1 and used when the next report is submitted."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print '%s is now running...' % currFunc

        if self.DEBUG: print "%s Connecting to database\n   ->%s" % (currFunc,self.CONNECT_STRING)
        # 2 # Open a connection to the db
        try:
            CONNECTION = pgdb.connect(self.CONNECT_STRING);

        except mdb.Error, e:
            print "%s Error %d: %s" % (currFunc,e.args[0],e.args[1])
            sys.exit(1)


        # Define query
        query_string ="SELECT batch FROM %s ORDER BY batch DESC LIMIT 1;" % self.DB_TABLE #This query finds out which batch was the last.

        # Execute query
        # Use try beause if the table is empty there is no batch number and that causes disgraceful exit.
        # Open a cursor to the connection.
        CURSOR = CONNECTION.cursor()
        if self.DEBUG: print "%s Sending query to the database for the batch number" % currFunc        
        CURSOR.execute(query_string)
        row = CURSOR.fetchone()
        # Use try beause if the table is empty there is no batch number and that causes disgraceful exit.
        try:
            current_batch=row[0]
            if self.DEBUG: print "%s Current batch number is: %s" % (currFunc, current_batch)        
        except:
            current_batch=0

        next_batch = current_batch + 1
        if self.DEBUG: print "%s Next batch number is: %s" % (currFunc, next_batch)        

        CURSOR.close()
        CONNECTION.close()

        return next_batch
    
    # ------------------ addReportToDB() ------------------ #

    def addReportToDB(self,station_info,station_status,next_batch):
        """Adds a checkomm report to the database."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print '%s is now running...' % currFunc

        if self.DEBUG: print "%s Connecting to database\n   ->%s" % (currFunc,self.CONNECT_STRING)
        # 2 # Open a connection to the db
        try:
            CONNECTION = pgdb.connect(self.CONNECT_STRING);

        except mdb.Error, e:
            print "%s Error %d: %s" % (currFunc,e.args[0],e.args[1])
            sys.exit(1)

        # All the data tailoring that happens here below should not be part of this module
        # The data fed to the module should be more unified.

        table = 'checkcomm'
        pos_lat = 0
        pos_lon = 0
        pos_alt = 0
        volt    = 0
        temp    = 0
        receiver= 0
        router  = 0


        # This transform is necessary for the structure of the checkomm table
        if station_status['connection']['router'] and station_status['connection']['router'] != 'N/N' and station_status['connection']['router'] != 'False':
            router = 1 

        if station_status['connection']['receiver'] and station_status['connection']['receiver'] != 'N/N' and station_status['connection']['receiver']     != 'False' :
            receiver = 1

        # This checks if the temp value is a number or not. If this fails then the default value, 0, just stays 0
        try:
            float(station_status['temp']['value'])
            temp = station_status['temp']['value'] 
        except:
            pass

        # This checks if the volt value is a number or not. If this fails then the default value, 0, just stays 0
        try:
            float(station_status['volt']['value'])
            volt = station_status['volt']['value'] 
        except:
            pass

        # This checks if latitude, longitude and altitude exist under 'value' and saves the values.
        try:
                float(station_status['position']['value']['longitude'])
                pos_lon = station_status['position']['value']['longitude']
        except:
            pass

        try:
                float(station_status['position']['value']['altitude'])
                pos_alt = station_status['position']['value']['altitude']
        except:
            pass

        try:
                float(station_status['position']['value']['latitude'])
                pos_lat = station_status['position']['value']['latitude']
        except:
            pass


        if station_info['proxy']['user'] == 'sil':
            router_ip = 'localhost'
        else:
            router_ip = station_info['router']['ip']

        # Next assemble a data variable 
        data = (
                station_info['station']['id'],
                router_ip,
                router,
                receiver,
                temp,
                volt,
                next_batch,
                pos_lat,
                pos_lon,
                pos_alt)

        
        if self.DEBUG: print "%s data to be sent: %s" % (currFunc,data)

        # Then, define a query
        insert_string ="INSERT INTO %s (sid, ip, rout_stat, recv_stat, recv_temp, recv_volt, batch, pos_lat, pos_lon, pos_alt) VALUES('%%s' , '%%s' , '%%s', '%%s', '%%s', '%%s', '%%s', '%%s', '%%s', '%%s');" % self.DB_TABLE

        if self.DEBUG: print "%s Just about to execute the 'put' command with string: %s " % (currFunc, insert_string)

        query_string = insert_string % data

        if self.DEBUG: print "%s Query string: %s" % (currFunc,query_string)

        # FINALLY : Execute query
        # Use try beause if the table is empty there is no batch number and that causes disgraceful exit.
        # Open a cursor to the connection.
        CURSOR = CONNECTION.cursor()

        if self.DEBUG: print "%s Sending data to database" % currFunc

        try:
            CURSOR.execute(query_string)

        except Exception , e:
            print 'ERROR:', e[0]
        
            

        CURSOR.close()
        CONNECTION.commit()
        CONNECTION.close()

    # ------------------ checkIfFileExists() ------------------ #
    def checkIfFileExists(self):
        """Checks if file metadata exists in the database."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if p_args['debug']: print '%s is now running...' % currFunc

    # ------------------ addFileToDB() ------------------ #
    def addFileToDB(self):
        """Adds file metadata to the database."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if p_args['debug']: print '%s is now running...' % currFunc




#!/usr/bin/python
"""
convllh(llh,radians=True)
extractfromGamitBakf(cfile, stations)
savedisp(dataDict,fname=None, header="")
"""
import subprocess

def run_cmd(check_cmd):
    ## Run command

    process = subprocess.Popen(check_cmd,shell=True,stdout=subprocess.PIPE)
    process.wait()

    proc_check_returncode = process.returncode
    proc_check_comm = process.communicate()[0].strip('\n'.encode())
    
    
    return proc_check_returncode,proc_check_comm

def run_syscmd(check_cmd,p_args):
    """
    """
    ## Run command
    import sys, re, subprocess, getopt, logging, copy, socket, types
    import gpstime

    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name
    if p_args['debug']: print('%s Starting ...' % currfunc)

    process = subprocess.Popen(check_cmd,shell=True,stdout=subprocess.PIPE)
    process.wait()

    proc_check_returncode = process.returncode
    proc_check_comm = process.communicate()[0].strip('\n')
    
    
    #(3)# Make desicions according to output...
    if p_args['debug']: print('%s process.returncode:' % currfunc, proc_check_returncode)
    if p_args['debug']: print(currfunc,'process.communicate():\n---------------\n', proc_check_comm,'\n-------------')
    if proc_check_returncode == 0:
        if p_args['debug']: print("%s Command went well.." % currfunc)
    elif proc_check_returncode == 255:
        if p_args['debug']: print("%s Timeout..." % currfunc)
    else:
        if p_args['debug']: print("%s Command failed... " % currfunc)

def conv2png(psFile,density=90,logo=None,logoloc="+780+0090"):
    """
    """
    import os


    fDir=os.path.dirname(psFile)
    fileName, Ftype = os.path.splitext(psFile)


    tmpFile=os.path.join(fDir, "tmp.png")
    pngFile="%s.%s" % (fileName,"png") 


    psCmd = "convert -density %d %s %s " % (density, psFile, tmpFile)
    run_cmd(psCmd)
        
    trCmd = "convert -trim %s %s " % (tmpFile,pngFile)
    run_cmd(trCmd)
    if os.path.isfile(tmpFile):
        os.remove(tmpFile)

    if logo:
        logoCmd = "composite -compose atop -gravity NorthEast -geometry +{0} -resize '1x1<' -dissolve 70% {1} {2} {3} ".format(logoloc, logo, pngFile, tmpFile)
        run_cmd(logoCmd)
        os.rename(tmpFile,pngFile)

        #logoCmd = "composite -compose atop -gravity NorthEast -geometry +830+0090 -resize '1x1<' -dissolve 50%" +  " %s %s %s " % (logo, pngFile, tmpFile)


def convpng2thum(pngFile):
    """
    """
    import os
    from PIL import Image

    img = Image.open(pngFile)
    filepr, Fend = os.path.splitext(pngFile)
    size = 128, 128
    img.thumbnail(size,Image.ANTIALIAS)
    img.save(filepr+'-small'+ Fend)



def run_netcmd(netcmd,port,station,p_args):

    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
    if p_args['debug']: print('%s Starting ...' % currfunc)

    conntype=station['conn_type'].split(',') # methods defined by conn_type

    #(1)# define how to handle differennt communication types
    if 'tunnel' in conntype:
        # direct ip connection through tunnels 
        if p_args['debug']: print('%s connecting through %s on port %s' % (currfunc,station['rout_ip'], station['recv_httpport']))
        netcmd = netcmd % '%s:%s' % (station['rout_ip'], port )
        if p_args['debug']: print('%s netcmd=' % currfunc,netcmd)

    elif 'sil' in conntype:
        # direct ip connection through sil computer using port forwarding
        if p_args['debug']: print('%s NetRS/NetR9 receiver on a SIL station' % currfunc)
        netcmd = netcmd % '%s:%s' % ('localhost',station['sil_httpport'])
        if p_args['debug']: print('%s netcmd=' % currfunc,netcmd)

    elif 'direct' in conntype:
        # direct ip connection 
        if p_args['debug']: print('%s Direct connection' % currfunc)
        netcmd = netcmd % station['recv_ip']
        if p_args['debug']: print('%s netcmd=' % currfunc,netcmd)
				
    elif 'serial' in conntype:
        if station['sil_computer']:
            pass
					
        elif '3G' in station['conn_type']:
            pass

    #(2)# run the command and return the output to proc_checktemp_comm
    proc_checktemp_returncode, proc_checktemp_comm=run_syscmd(netcmd, p_args)
    
    # Return
    return proc_checktemp_returncode, proc_checktemp_comm

def checkError(proc_check_returncode,proc_check_comm,searchres,p_args):

    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name
    if p_args['debug']: print('%s Starting ...' % currfunc)

    
    if proc_check_returncode == 0 and searchres: 
        if p_args['debug']: print( "%s all is OK" % currfunc)
        return 0
    elif 'ERROR' in proc_check_comm:
        if p_args['debug']: print( "%s proc_check_comm returned: %s" % (currfunc,proc_check_comm))
        return 11
    else:
        if p_args['debug']: print( "%s Something is wrong" % currfunc)
        print("ERROR in matching  proc_checksess_comm =\n",proc_check_comm,\
              " \nand checksess_returncode=",proc_check_returncode,\
              " \nsearchres=", searchres )
        #sys.exit(0)
        return 1

def changeDictKeys(session,chdict,p_args):
    
    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name

    if p_args['debug']: print('%s Starting inplace key change \ ...\n--\n Canging keys: %s\n--\n In: %s\n--\n To: %s' \
                               % (currfunc, chdict.keys(),session,chdict.values()))

    for sess in session:
        for key in set(chdict.keys()) & set(sess.keys()):
            sess[chdict[key]]=sess.pop(key)  

    return session

def checkPort(host,port,p_args):

    # variable port needs to be of type int
    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
    if p_args['debug']: print('%s Starting...' % currfunc)

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((host, port))
        s.shutdown(2)
        #if p_args['debug']:
        print("%s Success connecting to %s on port: %s" % (currfunc, host, port))
        return 0
    except:
        #if p_args['debug']:
        print("%s Cannot connect to %s on port: %s" % (currfunc, host, port))
        return 1


def getportdict(pref,station,p_args):  

    # Search string to check for defined ports
    #searchstr=re.compile('recv_(\w+)port') 
    searchstr=re.compile('\s*%s_(\w+)port=(\S+)\s*' % pref)
    stationstr= ' '.join([ '%s=%s' % (k,v) for k,v in station.items()])

    # Extracting the port definitions from recv_+port
    searchres=searchstr.findall(stationstr)
    recvport=dict(searchres)

    return recvport

def str_to_class(field):
    return getattr(sys.modules[__name__], field)
    #    try:
    #        identifier = getattr(sys.modules[__name__], field)
    #    except AttributeError:
    #        raise NameError("%s doesn't exist." % field)
    #    if isinstance(identifier, (types.ClassType, types.TypeType)):
    #        return identifier
    #    raise TypeError("%s is not a class." % field)




# -*- coding: utf-8 -*-
from __future__ import print_function
"""
Reading in Data Geophysical data from IMO


functions to read in data:
    def read_sil_data(start=None, end=None, area=None, frfile=None,
                      base_path="/mnt/sildata", fread="events.fps",
                      rcolumns=[], logging_level=logging.WARNING,
                      cum_moment=True, **kwargs):

functions for returning subsets:
    def seism_subset(data, start=None, end=None, min_lat=None, max_lat=None,
                     min_lon=None, max_lon=None, min_depth=None,
                     max_depth=None, min_mlw=None, max_mlw=None)

"""


#
# Functions returning data subsets.
#

# For seismic

def seism_subset(data, start=None, end=None,
                 min_lat=None, max_lat=None, min_lon=None, max_lon=None,
                 min_depth=None, max_depth=None, min_mlw=None, max_mlw=None,
                 dna=False, min_Q=None, max_Q=None, tstrf="%Y%m%d %H:%M:%S",
                 logger=__name__):
    '''Cuts out a subset of a dataframe assumes data.index is datetime index.'''
    import logging
    import pandas as pd

    # Handling logging
    module_logger = logging.getLogger(logger)

    if dna:
        data = data.dropna()

    start_ser = end_ser = data.index.notna()  # time, index
    min_lat_ser = max_lat_ser = data.index.notna()  # latitude
    min_lon_ser = max_lon_ser = data.index.notna()  # longitude
    min_depth_ser = max_depth_ser = data.index.notna()  # depth
    min_mlw_ser = max_mlw_ser = data.index.notna()  # Mlw
    min_Q_ser = max_Q_ser = data.index.notna()  # Q

    # making boolean filter arrays for all parameters
    if start is not None:
        start_ser = data.index > start
    if end is not None:
        end_ser = data.index < end
    if min_lat is not None:
        min_lat_ser = data["latitude"] > min_lat
    if max_lat is not None:
        max_lat_ser = data["latitude"] < max_lat
    if min_lon is not None:
        min_lon_ser = data["longitude"] > min_lon
    if min_lon is not None:
        max_lon_ser = data["longitude"] < max_lon
    if min_depth is not None:
        min_depth_ser = data["depth"] > min_depth
    if max_depth is not None:
        max_depth_ser = data["depth"] < max_depth
    if min_mlw is not None:
        if "MLW" in data:
            min_mlw_ser = data["MLW"] > min_mlw
        elif "ML" in data:
            min_mlw_ser = data["ML"] > min_mlw

    if max_mlw is not None:
        if "MLW" in data:
            max_mlw_ser = data["MLW"] < max_mlw
        elif "ML" in data:
            max_mlw_ser = data["ML"] < max_mlw

    if min_Q is not None:
        if "Q" in data:
            min_Q_ser = data["Q"] > min_Q

    if max_Q is not None:
        if "Q" in data:
            max_Q_ser = data["Q"] < max_Q

    # Applying all the filters in one call

    data = data[start_ser & end_ser &
                 min_lat_ser & max_lat_ser & 
                 min_lon_ser & max_lon_ser & 
                 min_depth_ser & max_depth_ser & 
                 min_mlw_ser & max_mlw_ser &
                 min_Q_ser & max_Q_ser]

    module_logger.info("Input time period: ({0}, {1}). Range in DataFrame: ({2}, {3})"
                        .format(start, end,  data.index.min(), data.index.max()) )
    module_logger.info("Selected latitude range: ({0}, {1}). Range in DataFrame: ({2:.3f}, {3:.3f})"
                        .format( min_lat, max_lat, data.latitude.min(), data.latitude.max()) )
    module_logger.info("Selected longitude range: ({0}, {1}). Range in DataFrame: ({2:.3f}, {3:.3f})"
                        .format( min_lon, max_lon, data.longitude.min(), data.longitude.max()) )
    module_logger.info("Selected depth range: ({0}, {1}). Range in DataFrame: ({2:.3f}, {3:.3f})"
                        .format( min_depth, max_depth, data.depth.min(), data.depth.max()) )

    if "MLW" in data:
        module_logger.info("Selected magnitude range: ({0}, {1}). Range in DataFrame: ({2:.2f}, {3:.2f})"
                            .format( min_mlw, max_mlw, data.MLW.min(), data.MLW.max()) )
    elif "ML" in data:
        module_logger.info("Selected magnitude range: ({0}, {1}). Range in DataFrame: ({2:.2f}, {3:.2f})"
                            .format( min_mlw, max_mlw, data.ML.min(), data.ML.max()) )
     
    if "Q" in data:
        module_logger.info("Selected quality range: ({0}, {1}). Range in DataFrame: ({2:.2f}, {3:.2f})"
                            .format( min_Q, max_Q, data.Q.min(), data.Q.max()) )

    module_logger.info("Number of earquakes: {0}".format( data.size) )
    module_logger.info("dataframe First and Last lines:\n"  + str(data.iloc[[0,-1]]))
    #module_logger.info("Latidues: {}".format( (min_lat_list and max_lat_list) ) )
    #data = data[start.strftime(tstrf):end.strftime(tstrf)]

    return data


def distalongline():
    """
    """


#
# Functions reading in data from data sources
#

def read_sil_file(dfile, duplicate=True, logger=__name__ ):
    """
    Open files from in a sil directory returning its content in a dataframe
    """

    import pandas as pd
    import logging
    import os


    # Handling logging
    module_logger = logging.getLogger(logger)

    module_logger.debug("Reading file {}:  ".format(dfile))

    fread = os.path.basename(dfile)
    
    # defining file structures of sil files
    if fread == "lib.mag":
        module_logger.info("reading data from  file {}:  ".format(dfile))

        # athuga nota num_of_st_pha
        # min_dist_sta distance to closest station
        columns = ["num", "date", "time", "latitude", "longitude", "depth", "MLW", "ML", "num_of_st_ML", 
                   "minMl", "maxMl", "num_of_st_pha", "num_of_P-pha", "num_of_S-pha", "num_of_polarities", 
                   "Ddata_time", "Dlatitude", "Dlongitude", "Ddepth", "w_rms_res", "min_dist_sta", 
                   "max_azim_gap", "vel_modelk", "event_type"]
        col_dtypel = ['int64', 'float64', 'float64', 'float64', 'float64', 'float64', 'int64',
                     'float64', 'float64', 'int64', 'int64', 'int64',
                     'int64', 'float64', 'float64', 'float64', 'float64', 'float64',
                     'float64', 'float64', 'int64', 'object']
        col_with=[ (0, 4), (4, 13), (13, 24), (24, 33), (33, 43), (43, 50), (50, 56), (56, 62), (62, 67), 
                   (67, 73), (73, 79), (79, 82), (82, 85), (85, 88), (88, 92), 
                   (92, 98), (98, 104), (104, 110), (110, 116), (116, 123), (123, 130), 
                   (130, 136), (136, 138), (138, 141)]

        

    elif fread=="events.fps":
        module_logger.info("reading data from  file {}:  ".format(dfile))

        columns =    [ "num", "date", "time", "latitude", "Dlatitude", "longitude", "Dlongitude", "depth", "Ddepth",
                       "MLW", "id", "seismic_moment", "ev_mag_ATH", "event_type", "sh_min", "sh_max", "azim_comp", 
                       "relative_size", "shearwave_corner_freq", "fault_radius", "stress_drop", "peak_slip", "source_file",
                       "strike_a_agr", "dip_a_agr", "slip_a_agr", "strike_b_agr", "dip_b_agr", "slip_b_agr", 
                       "num_of_polarities", "num_of_amplitudes", "unknown_pacc","is_significant"]
        col_dtypel = ['int64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 
                      'float64', 'object', 'float64', 'float64', 'object', 'float64', 'float64', 'float64', 
                      'float64', 'float64','int64', 'float64', 'float64', 'object', 
                      'int64', 'int64','int64', 'int64', 'int64', 'int64', 
                      'int64', 'int64', 'float64', 'int64']


    elif fread=="events.lib":
        module_logger.info("reading data from  file {}:  ".format(dfile))

        columns=[ "num", "date", "time", "latitude", "longitude", "depth", "MLW", "event_type", "id", "sh_min", "sh_max", "azim_comp", "num_of_polarities"]
        col_dtypel = ['int64', 'float64', 'float64', 'float64', 'float64', 'object', 'object', 'float64', 'float64', 'float64', 'int64']
        col_with=[ (0, 4), (4, 13), (13, 24), (24, 33), (33, 43), (43, 50), (50, 55), (55, 59), (59, 92), (92, 101), (101, 108), (108, 115), (115, 118)]


    elif fread=="cleanaut.mag" or fread=="aut.mag":
        module_logger.info("reading data from  file {}:  ".format(dfile))

        columns = ["num", "date", "time", "latitude", "longitude", "depth", "ML", "Q", "id"]
        col_dtypel = ['int64', 'float64', 'float64', 'float64', 'float64', 'float64',  'object']
        col_with=[ (0, 4), (4, 13), (13, 22), (22, 30), (30, 39), (39, 45), (45, 51), (51, 57), (57, 89)]

    else: # if file not regoginced raise an error
        module_logger.error("Unrecognised file name {}:  ".format(dfile))

        raise IOError 

    try:
        module_logger.info("Trying read columns {} from file {}:  ".format(columns, dfile))

        rcolumns = [columns[0]] + columns[3:]
        col_dtype = dict(zip(rcolumns, col_dtypel))

        if fread=="events.fps":
            tmp = pd.read_csv(dfile,  sep='\s+',  header=None, names=columns, dtype=col_dtype, infer_datetime_format=True,
                    parse_dates={"date_time": ['date', 'time']}, index_col="date_time")
            tmp=tmp.dropna()
        else:
            tmp = pd.read_fwf(dfile, header=None, colspecs=col_with, dtype=col_dtype, infer_datetime_format=True
                    names=columns, parse_dates=[['date', 'time']], index_col="date_time")
            tmp=tmp.dropna()

        if duplicate == False:
            tmp = tmp[~tmp.duplicated()]
        
        if tmp.empty:
            # If empty file, will have columns 'date' and 'time' in empty
            # DataFrame as pandas will not apply parse_dates/index_col,
            # must drop them to avoid adding empty columns to final DataFrame.
            # Example: /mnt/sk2/1993/jan/29/events.fps (no event that day)
            tmp = pd.DataFrame(columns=rcolumns)

    
    except ValueError as e:        
        module_logger.info("While reading File {0}: exception of type {1} occured, returning an empty DataFrame: ".format( dfile, type(e).__name__ ) + str(e))

    except  IOError as e:
        module_logger.info("While reading File {0}: exception of type {1}:  ".format( dfile, type(e).__name__ ) + str(e))
        raise IOError

    except BaseException as e:
        module_logger.info("While reading File {0}: exception of type {1}: ".format( dfile, type(e).__name__ ) + str(e), exc_info=True)
        raise


    return tmp


def read_automatic( ddir, aut=True, repl_mlw=False, logger=__name__ ):
    """
    """
    import pandas as pd
    import logging
    import os


    # Handling logging
    module_logger = logging.getLogger(logger)


    rcolumns= ['latitude', 'longitude', 'depth', 'MLW', 'Q', 'id', 'seismic_moment', "MW"]

    eventfps = pd.DataFrame(columns = rcolumns)
    cleanaut = pd.DataFrame(columns = rcolumns)
                
    dfile = os.path.join( ddir,"events.fps")
    module_logger.debug("Reading file {}:  ".format(dfile))
    
    if aut == "only":
        pass
    else:
        try: 
            eventfps=read_sil_file(dfile, logger=logger)
       
            eventfps["MW"] = eventfps.seismic_moment.apply(MW)
            eventfps["Q"] = 99.99
            eventfps = eventfps[rcolumns]

        except AttributeError as e:
            module_logger.info("missing columns from dataFrame:\n"  + str(eventfps) )
            eventfps = pd.DataFrame(columns = rcolumns)

        except  pd.io.common.EmptyDataError as e:
            module_logger.info("While reading File {}:  ".format(dfile) + str(e))
            eventfps = pd.DataFrame(columns = rcolumns)

        except  IOError as e:
            module_logger.info("While reading File {}:  ".format(dfile) + str(e))
            eventfps = pd.DataFrame(columns = rcolumns)


    

    if aut:
        dfile = os.path.join( ddir,"cleanaut.mag")
        module_logger.debug("Reading file {}:  ".format(dfile))
    
    
        try:
            cleanaut = read_sil_file( dfile, logger=logger )
        
        except  IOError as e:
            module_logger.info("While reading File {}:  ".format(dfile) )
            dfile = os.path.join( ddir,"aut.mag")
            module_logger.info("Will try  {}:  ".format(dfile))

            try:
                cleanaut=read_sil_file(dfile, logger=logger )

            except  IOError as e:
                module_logger.info("While reading File {}:  ".format(dfile) + str(e))
                cleanaut = pd.DataFrame(columns = rcolumns)

            except NotImplementedError as e:
                module_logger.info("While reading File {}:  ".format(dfile) + str(e))
                cleanaut = pd.DataFrame(columns = rcolumns)


        
        #cleanaut = read_sil_file( dfile, logging_level=logging.INFO )

        if cleanaut.empty:
            module_logger.info("dataframe is empty:\n"  + str(cleanaut) )
            cleanaut = pd.DataFrame(columns = rcolumns)

        else:
            cleanaut = cleanaut.rename(columns={"ML":"MLW"})
            cleanaut['seismic_moment'] = float('nan')
            cleanaut['MW'] = float('nan')
            cleanaut = cleanaut[rcolumns]
                    
        data  = pd.concat([eventfps, cleanaut[~cleanaut['id'].isin(eventfps['id'])]])
    
    else:
        data = eventfps

    return data

def MW(S_moment):
    """
    Calculate moment magnitude from seismic moment
    """
    from numpy import log10

    return log10(S_moment)*2/3 - 6.03 

def pgverror(r,M): 
    """
    calculate relative difference if skipping nonlinear term in PGV model for calulating MW
    See:

    Attenuation relations for near- and far-field peak ground motion (PGV, PGA) 
    and new magnitude estimatesforlargeearthquakesin SW-IcelandforlargeearthquakesinSW-Iceland
    VÍ 2009-012, ISSN 1670-8261: page: 25 eq C: 
    derive
    """
    
    from numpy import log10
    
    return ( log10( r+0.00299*10**(0.621*M)) - log10(r) ) / log10(r) 

def M_PGV(x, mu=0.000001):
    """
    Return Moment magnitude estimation based on PGV model derived in 
    
    Attenuation relations for near- and far-field peak ground motion (PGV, PGA) 
    and new magnitude estimatesforlargeearthquakesin SW-IcelandforlargeearthquakesinSW-Iceland
    VÍ 2009-012, ISSN 1670-8261: page: 25 eq C: 
    
    expect pgv in m/s
    and    r in km

    """
    from numpy import log10

    

    return 1/1.05 * ( 1.69*log10(x[0]) +  log10( mu*x[1] ) + 4.96 )


##
# modules to extract PGV from Alert and distance in order to estimate Mw from PGV equation
##
def read_evb(filename, logger="__name__"):
    """
    """

    from datetime import datetime as dt
    import pandas as pd
    import logging


    # Handling logging
    module_logger = logging.getLogger(logger)

    columns = [ "sta", "arr.", "time", "res.",  "weight", "dist" , "azi", "az.obs",  "err", "number"]
    col_dtypel = ['str', 'str', 'int64', 'int64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64', 'float64']

    col_with=[ (0, 4), (4, 6), (6, 18), (18, 24), (24, 30), (30, 36), (36, 42), (42, 49), (49, 55), (55, 62) ]
    
    parser = lambda time: dt.strptime(time, '%H %M %S.%f')
    #parse_dates={"time": ['hour', 'minute', 'sec']},
    data = pd.read_fwf( filename, skiprows=5, skipfooter=1, colspecs=col_with, names=columns, parse_dates=["time"], date_parser=parser, index_col="sta")
    data['time'] = pd.to_datetime(data.time).dt.time

    return data


def read_alert(filename, logger="__name__"):
    """
    """

    import pandas as pd
    import logging

    # Handling logging
    module_logger = logging.getLogger(logger)

    rcolumns = ["lon", "lat", "time", "pgv", "sta", "fb"]
    col_dtypel = ['float64', 'float64', 'float64', 'float64', 'str', 'float64']

    col_dtype = dict(zip(rcolumns, col_dtypel))

    data = pd.read_csv(filename, skiprows=1, sep="\s+", usecols=[0,1,2,3,4,5],
                       names=rcolumns, dtype=col_dtype,  index_col="sta")
    data['time'] = data['time'].div(100)  # change from samples to seconds
    return data


def downl_PGV_hraun(inpdate=None, tmp_path=None, dstr='/%Y/%b/%d',
                    logger=__name__, rm_files=False):
    """
    """

    import os
    import locale
    import logging
    import glob

    from datetime import date
    import urllib.request as ur

    import gtimes.timefunc as gt

    # Handling logging
    module_logger = logging.getLogger(logger)

    # changing locals so the system will recoginse month folders correctly (i.e des -> dec)
    loc = locale.getlocale()
    locale.setlocale(locale.LC_TIME, ('en_US', 'UTF-8'))

    # defing default period to extract
    # dstr='%Y-%m-%d %H:%M:%S'
    if inpdate == None:
        inpdate = date.today()

    elif type(inpdate) is str:
        inpdate = gt.toDatetime(inpdate, dstr)

    alert_url = "http://hraun.vedur.is/ja/alert/"
    dpath = inpdate.strftime("%Y/%b/%d/").lower()

    if tmp_path is None:
        tmp_path = "/tmp/alert/"

        # making sure download directoty exists and is empty
        try:
            os.makedirs(tmp_path)

        except FileExistsError as e:
            module_logger.info("directory {} exists:  ".format(tmp_path) + str(e), exc_info=True)

            if rm_files:
                files = glob.glob(tmp_path+'*')
                for f in files:
                    os.remove(f)
            else:
                raise

        except FileNotFoundError as e:
            module_logger.info("Can not create directory   {}:  ".format(tmp_path) + str(e), exc_info=True)

    i = 0
    while True:
        i += 1
        filename = "{0:03d}.event".format(i)
        try:
            ur.urlretrieve(alert_url + dpath + filename, os.path.join(tmp_path, filename))
        except:
            break

    # restoring locale settings
    locale.setlocale(locale.LC_TIME, loc)

    return tmp_path + "/"


def find_alert(event_id, sil_path="/mnt_data/sildata", pre_path="/tmp", eps=30, logger=__name__):
    """
    Get one SIL event evb, download alertMap events, return matching data.

    Parameters
    ----------
    event_id : str
        SIL event id = path to evb|eve|ana files.
        Example: "/2020/jun/25/00:05:00/09:33:883"
    sil_path : str, optional
        Path to SIL top dir which contains the year dirs. The default is
        "/mnt_data/sildata".
    pre_path : str, optional
        Path to local alertMap output dirs. The default is "/tmp".
        Will download XXX.event files to here and then parse them from there.
        Original location: "hellir:/var/www/sil.vedur.is/alert"
    eps : int, optional
        Time tolerance in seconds for associating events from SIL and
        alertMap. The default is 30.
    logger : str, optional
        Name of logger object. The default is __name__ (current method name).

    Returns
    -------
    dist_pgv : pandas.DataFrame
        Has index = station code, columns = dist, pgv, MW_pgv; for stations
        common in SIL *.evb and alertMap *.event files.
    """
    import logging
    import os
    import locale
    import csv
    import glob

    from datetime import datetime as dt
    from datetime import timedelta
    from pathlib import Path
    from shutil import rmtree

    loc = locale.getlocale()
    locale.setlocale(locale.LC_TIME, ('en_US', 'UTF-8'))

    # Handling logging
    module_logger = logging.getLogger(logger)

    # inpdate=dt.strptime(event_id[:12],"/%Y/%m/%d")
    # dpath=inpdate.strftime("%Y/%b/%d/").lower()

    tmp_path = os.path.join(pre_path, event_id[:12])

    # making sure download directoty exists and is empty
    try:
        os.makedirs(tmp_path)

    except FileExistsError as e:
        module_logger.info("directory {} exists:  ".format(tmp_path) + str(e), exc_info=True)

    except FileNotFoundError as e:
        module_logger.info("Can not create directory   {}:  ".format(tmp_path) + str(e), exc_info=True)
        raise

    locale.setlocale(locale.LC_TIME, loc)

    tmp_dir = sil_path + event_id
    filename = os.path.join(tmp_dir + ".evb")

    EVB = read_evb(filename, logger="__name__")
    # Download all alert files from the day of event_id and return the download path
    tmp_path = downl_PGV_hraun(inpdate=event_id[:12], tmp_path=tmp_path, logger=__name__, rm_files=True)

    files = glob.glob(tmp_path + '*')
    fields = {}
    for f in files:
        with open(f, 'r') as infile:
            reader = csv.DictReader(infile)
            fields[ reader.fieldnames[0].split()[0]] = reader.fieldnames[0].split()[4:8] 

    eps = timedelta(seconds=eps)
    window_min = (dt.combine(dt.today(), EVB.time.min()) - eps).time()
    window_max = (dt.combine(dt.today(), EVB.time.max()) + eps).time()
    for key, value in fields.items():
        event_found = window_min < dt.strptime(value[-1], "%H:%M:%S").time() < window_max

        module_logger.debug("Searching for event in {0}.event -> {1}, Found: {2}".format(key, value, event_found) )

        if event_found:
            use_file = tmp_path + key + ".event"
            module_logger.info("Found event in  -> {0}".format(use_file))

            break

    try:
        module_logger.warning("Attempting to fecth PGV values from -> {0}".format(use_file))
        PGV = read_alert(use_file)
        common_sta = list(set(PGV.index) & set(EVB.index))

        module_logger.warning("List of stations: {0}".format(common_sta))

        PGV = PGV.loc[common_sta]
        EVB = EVB.loc[common_sta]

        dist_pgv = EVB['dist'].to_frame().join(PGV['pgv'].to_frame())
        dist_pgv['MW_pgv'] = dist_pgv.apply(M_PGV, axis=1)
        dist_pgv.sort_values('dist', inplace=True)

    except:
        module_logger.warning("No alert file found for event -> {0}".format(event_id), exc_info=True)
        dist_pgv = None

    # remove the downloaded files and all the directories
    try:
        # rmtree( Path(tmp_path).parents[1] )
        pass
    except:
        module_logger.warning("Directory {0} not found".format(tmp_path))

    return dist_pgv


##
# ----------------------------------------------------------------------------
##

def removedubl(data, idcol=True, timetag=True):
    """
    """

    import numpy as np
    
    if idcol == True:
        tmp = data[data.id.duplicated(keep=False)].sort_values(['id','Q'], ascending=False)
        data.drop((tmp[tmp.id.duplicated()].index), inplace=True)

    if timetag == True:
        # adding the number 0 ... len(data) to the dataFrame for identification
        print(data.index.duplicated().any())
        print(data[data.index.duplicated(keep=False)])
        if data.index.duplicated().any():
            ilocation = data.reset_index().index
            data['ilocation'] = ilocation # tmp column to id correct rows
    
   
            # find all duplicate datetime tags and selct the row with lower "Q" to be removed
            print(data[data.index.duplicated(keep=False)].reset_index().sort_values(by=['Q']).set_index('index'))
            #tmp = data[data.index.duplicated(keep=False)].reset_index().sort_values(by=['index', 'Q']).set_index('index') 
            tmp = data[data.index.duplicated(keep=False)].reset_index().sort_values(by=['Q']).set_index('index') 
            iddrop = tmp[tmp.index.duplicated(keep='last')]['ilocation']
            print(iddrop)
    
            idx = ~np.ones(len(ilocation), dtype=bool) # True array
            idx[iddrop] = True # identifing colums to drop
            data.drop( data[idx].index, inplace=True) # droping dublicated time tags
            data.drop( ['ilocation'], inplace=True, axis=1 )# removing the tmp column)

    return data

def read_sil_data(
        start="1990-01-01", end=None,
        frfile=False, base_path="/mnt_data/sildata",
        fread="default", fname=None, rcolumns=[],
        aut=True, aut_per=30, repl_mlw=False, dna=False, duplicate=False,
        logger=__name__, logging_level=None, cum_moment=True, **kwargs):
    """
    Read earthquake parameters from IMO's SIL data structure.

    Parameters
    ----------
    start : string or datetime or int, optional
        Start of selected time range. Either string in ISO dateformat, or int
        days before end. The default is 1990-01-01.
    end : string or datetime, optional
        End of selected time range. The default is now.
    frfile : bool, optional
        Read dataframe from pickle/parquet file? If true, will use fname as
        filepath. The default is False.
    base_path : str, optional
        Path to SIL top dir which contains the year dirs,
        not used if frfile=True. The default is "/mnt_data/sildata".
    fread : str, optional
        Which type of SIL file: "lib.mag" or "events.fps",
        not used if frfile=True. The default is "events.fps".
    fname : str, optional
        Path to pickle/parquet file if frfile=True. The default is None.
    rcolumns : str, optional
        Columns to return: default selection if None or "default",
        else get all available. The default is None.
    aut : bool or str, optional
        Only used if fread="default". If aut=True, include automatic solutions
        from cleanout.mag. The default is True.
    aut_per : int, optional
        Set for how many days back from end automatic solutions to be included.
        Either an int or "all" can be set. The default is 30.
    repl_mlw : bool, optional
        Not used. The default is False.
    dna : bool, optional
        If True, drop Nan rows of output dataframe.
        TODO: Applied regardless for all files in read_sil_file(), fix it.
        The default is False.
    duplicate : bool, optional
        If true, remove duplicate entries using pandas.duplicates().
        The default is False.
    logger : str, optional
        Name of logger object. The default is __name__ (current method name).
    logging_level : str, optional
        Force set logging level by string:
            "NOTSET", "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL".
        The default is None.
    cum_moment : bool, optional
        Whether to calculate and output cumulative moment as well.
        The default is True.
    **kwargs : all float
        More keys can be added to limit selection:
            min_lat, max_lat, min_lon, max_lon,
            min_depth, max_depth, min_mlw, max_mlw,
            min_Q, max_Q

    Returns
    -------
    pandas.DataFrame. Its index is event origin datetime, its columns
    are by default "latitude", "Dlatitude", "longitude", "Dlongitude",
    "depth", "Ddepth", "MLW", "seismic_moment", but can be different
    based on INPUT fread and rcolumns.
    """
    
    # Imports
    import os
    import logging
    import time
    
    from datetime import datetime as dt
    from datetime import timedelta

    import pandas as pd

    import gtimes.timefunc as gt

    # defining default period to extract
    if end is None:
        end = dt.now()
    elif isinstance(end, str):
        end = pd.to_datetime(end)
    if isinstance(start, int):
        start = abs(start)
        start = gt.currDatetime(-start,refday=end)
    elif isinstance(start, str):
        start = pd.to_datetime(start)


    # Handling logging
    logging.basicConfig()
    module_logger = logging.getLogger(logger)
    module_logger.setLevel(logging.WARNING)
    if logging_level:
        module_logger.debug('Set logger level: {}'.format(logging_level))
        module_logger.setLevel(
            {"NOTSET": logging.NOTSET,
             "DEBUG": logging.DEBUG,
             "INFO": logging.INFO,
             "WARNING": logging.WARNING,
             "ERROR": logging.ERROR,
             "CRITICAL": logging.CRITICAL}.get(
                logging_level.upper(), "WARNING"))

    # special formating of months used in some file structures #b lowercase i.e jan, feb, ...
    dstring="{0}/%Y/#b/%d".format(os.path.join(base_path))
    # standard formating of month %b uppercase i.e Jan, Feb
    standard_dstring="{0}/%Y/%b/%d".format(os.path.join(base_path))
    # time formating
    tstrf="%Y%m%d %H:%M:%S"

     
    auto_logg='default'
    df_tmp_list=[]
    start_run = intermediate_time = time.time()
    if frfile:
        if not fname:
            fname="sil-data.pik"
        data = read_pd_file(fname)

    else:
        # check basepath access and contents:
        if not os.path.isdir(base_path):
            raise Exception('SIL basepath not a directory: {}'.format(base_path))
        if not os.access(base_path, os.R_OK):
            raise Exception('SIL basepath not readable: {}'.format(base_path))
        if not os.listdir(base_path):
            raise Exception('SIL basepath is empty dir: {}'.format(base_path))

        # Creating a list of files to open
        dlist=gt.datepathlist(dstring,'0.8D',start,end, closed=None )
        
        ryear=int(start.strftime("%Y"))
        for ddir in dlist:
            module_logger.debug("Reading from directory {}:  ".format(ddir))
            
            # date from the string
            rdate = dt.strptime(ddir, standard_dstring)
            # file to read
            dfile = os.path.join(ddir,fread) 
            lyear=int(rdate.strftime("%Y"))
            # Printing out info on progress
            if lyear == ryear:
                module_logger.warning("------------ working on year {} ----------:".format(ryear))
                module_logger.warning("Reading files from:  {}:".format(os.path.join( "/", *ddir.split(os.sep)[:-2] )))

            # To return standard  output. bulleting data partly or all from automatic processing
            # Returning ['latitude', 'longitude', 'depth', 'MLW', 'Q', 'id', 'seismic_moment', "MW"]
            # MLW is ML if the event is from automatic processing.
            # MW = 2/3*log10(seismic_moment) - 6.03
            # MLW is replaced by MW if MLW > repl_mlw, (default not replace) 

            HAVE_FILE = True
            if fread == 'default':
                if aut:
                    if aut_per == "all":
                        tmp = read_automatic( ddir, aut=True, repl_mlw=repl_mlw, logger=auto_logg )

                    elif rdate > (end - timedelta(days = aut_per)):
                        tmp = read_automatic( ddir, aut=True, repl_mlw=repl_mlw, logger=auto_logg )

                    else:
                        tmp = read_automatic( ddir, aut=False, repl_mlw=repl_mlw, logger=auto_logg )

                else:
                    tmp = read_automatic( ddir, aut=False, repl_mlw=repl_mlw, logger=auto_logg )

            else:
                try:
                    if os.path.isfile(dfile):
                        tmp=read_sil_file(dfile)
                    else:
                        module_logger.info("File does not exist: {}".format(dfile))
                        HAVE_FILE = False

                except  IOError as e:
                    module_logger.info("While reading File {}:  ".format(dfile) + str(e), exc_info=True)
                    HAVE_FILE = False

            if HAVE_FILE:
                df_tmp_list.append(tmp)

            # Printing out info on progress

            if lyear == ryear:
                delta_time=time.time()- intermediate_time
                intermediate_time = delta_time + intermediate_time
                module_logger.warning("Total time working data from {0} is {1:.3f} s: ".format( ryear, delta_time ))
                ryear += 1
                #print(module_logger)


        data = pd.concat(df_tmp_list)
        module_logger.warning("Total time in for looping through all data: {0:.3f} s: ".format( time.time() - start_run ))
       
        #if data.empty:
        #    module_logger.exception("Empty DataFrame: {}:".format( data))
        #    raise ValueError
       
        
        if aut_per == "all" and fread == "default":
            module_logger.info( "using automatic events for all events not in events.fps" )
        elif fread == "default":
            module_logger.info( "Using automatic events for all events not in events.fps after {}:  ".format(end - timedelta(days = aut_per))) 
        else:   
            module_logger.info( "Reading from file {}:  ".format(dfile)) 

    # only returning subset of columns
    if data is not None and isinstance(data, pd.DataFrame) and not data.empty:
        if rcolumns == "default" or rcolumns == []: # Different options of column return
            if fread=="lib.mag":
                rcolumns=["latitude", "Dlatitude", "longitude", "Dlongitude", "depth", "Ddepth", "MLW", "ML"]

            elif fread=="events.fps":
                rcolumns=[ "latitude", "Dlatitude", "longitude", "Dlongitude", "depth", "Ddepth", "MLW", "seismic_moment"]
            
            elif fread=="cleanaut.mag":
                rcolumns=[ "latitude", "longitude", "depth",  "ML", "Q"]
            
            else:
                rcolumns=data.columns.to_list()

            module_logger.info( "Colums {} selected from file {}".format(rcolumns, fread) )

        else:
            if rcolumns == "all":
                rcolumns=data.columns.to_list()
                
            else:
                rcolumns=list(set(rcolumns) & set(data))
                module_logger.info( "Colums {} selected from file {}".format(rcolumns, fread) )



        if duplicate == False:
            data = data[~data.duplicated()]

        # in case of duplicate id
        if "Q" in data.columns and "id" in data.columns:
            data = removedubl(data,timetag=False)

           

        data = data[rcolumns]
        #data = data[start.strftime(tstrf):end.strftime(tstrf)]
        data = seism_subset(data, start=start, end=end,
                            **kwargs, dna=dna, logger=auto_logg)

        
        # Return cumulative moment 
        if cum_moment and  ('seismic_moment' in data) :
            data['cum_moment'] = data['seismic_moment'].cumsum()

        # For debugging: logging
        module_logger.info("Dataframe columns:\n" + str(data.columns) + "\n")
        module_logger.info("Input time period: ({}, {})".format(start, end))
        module_logger.info("dataframe First and Last lines:\n"  + str(data.iloc[[0,-1]]))
        module_logger.debug("Dataframe shape: {}".format( str(data.shape) )) 
        module_logger.debug("dataframe types:\n" + str(data.dtypes) + "\n" )

    else:
        module_logger.warning("dataframe is empty:\n"  + str(data) )
    
    module_logger.debug("dataframe File list:  " + str(dlist) )
    module_logger.debug("String to produse directory list:  " + dstring )

    module_logger.info("Time since for loop finished {0:.3f} s: ".format( time.time() - intermediate_time ))
    module_logger.info("Total time {0:.3f} s: ".format( time.time() - start_run ))

    return data


def read_pd_file(fn, filetype=None):
    """
    Read pandas dataframe saved as pickle or parquet file.

    This method is for convenience to read any of the two formats and any
    compression without having to specify the pandas read method.
    Method copy-pasted from shiver.u_io 2020-06-24 10:46 by tim.

    Parameters
    ----------
    fn : string
        Path to file, expected to be only pandas dataframe.
    filetype : string, optional
        Can specify if 'pickle' or 'parquet'. The default is None.

    Returns
    -------
    pandas.DataFrame
        Object which had been stored as file.
    """
    import pandas as pd
    pickle_opt = ['pickle', 'pkl', 'pik']
    parquet_opt = ['parquet', 'prq', 'pqt']
    ext_pkl_comp = ['.gz', '.bz2', '.xz']
    ext_prq_comp = ['.zst', '.gz', '.lz4', '.snp', '.lzo']
    ext_pkl = ['pickle', 'pkl', 'pik']
    ext_prq = ['parquet', 'prq', 'pqt']
    # if filetype specified by user:
    if filetype:
        if filetype.lower() in pickle_opt:
            return pd.read_pickle(fn)
        elif filetype.lower() in parquet_opt:
            return pd.read_parquet(fn)
        else:
            raise RuntimeError(
                "Given filetype '{}' unavailable.".format(filetype))
    fl = fn.lower()
    # if file extension matches common ones for (compressed) pickle/parquet:
    if any([fl.endswith(x) for x in ext_pkl_comp]):
        e2 = fl.split('.')[-2]
        if any([e2 == x for x in ext_pkl]):
            try:
                return pd.read_pickle(fn)
            except Exception as e:
                print('Warning: read as pickle (based on file-ext.) failed.')
                raise e
    if any([fl.endswith(x) for x in ext_prq_comp]):
        e2 = fl.split('.')[-2]
        if any([e2 == x for x in ext_prq]):
            try:
                return pd.read_parquet(fn)
            except Exception as e:
                print('Warning: read as parquet (based on file-ext.) failed.')
                raise e
    if any([fl.endswith(x) for x in ext_pkl]):
        try:
            return pd.read_pickle(fn)
        except Exception as e:
            print('Warning: read as pickle (based on file-ext.) failed.')
            raise e
    if any([fl.endswith(x) for x in ext_prq]):
        try:
            return pd.read_parquet(fn)
        except Exception as e:
            print('Warning: read as parquet (based on file-ext.) failed.')
            raise e
    # if no info given and file ext not typical, try both:
    try:
        return pd.read_pickle(fn)
    except:
        pass
    try:
        return pd.read_parquet(fn)
    except Exception as e:
        print('Warning: read as pickle and parquet failed.')
        raise e


##
# functions used for real time fixes of magnitude on bigger quakes
##

def read_station_loc(fileloc):
    """
    Read a net.dat and put into dataframe.

    Parameters
    ----------
    fileloc : str
        system location of file ex: /home/liljajo/Documents/net.dat

    Returns
    -------
    pandas dataframe
    """
    import pandas as pd

    columns = ["sta", "latitude", "longitude", "height"]
    col_dtypel = ['object', 'float64', 'float64', 'float64']
    col_with = [(0, 4), (4, 13), (13, 23), (23, 29)]
    col_dtype = dict(zip(columns, col_dtypel))
    tmp = pd.DataFrame(columns=columns)
    tmp = pd.read_fwf(fileloc, header=None, colspecs=col_with, dtype=col_dtype,
                      infer_datetime_format=True, names=columns)
    tmp = tmp.dropna()
    return tmp


def check_event_fit(location, SIL_time, PGV_time, PGV):
    """
    Find and compare arrival times of PGV to given criteria.

    Also filters for stations further than 230 km away, LE1 and ada.

    Parameters
    ----------
    location : list
        list of latitude and longitude of origin from SIL
    SIL_time: datetime
        datetime object of SIL origin time
    PGV_time: datetime
        datetime object from header of event file
    PGV: dataframe
        with index: "sta", columns: "fb", "lon", "lat", "time", "pgv"

    Returns
    -------
    True: boolean and PGV_fit: dataframe if enough sta fit within

    False: boolean and PGV_fit: dataframe if not enough sta fit
    """
    import pandas as pd
    from geopy import distance
    from datetime import datetime as dt
    from datetime import timedelta

    # add distances from SIL origin to PGV dataframe
    latitudes = PGV['lat'].tolist()
    longitudes = PGV['lon'].tolist()
    coordinates = list(map(lambda x, y: (x, y), latitudes, longitudes))
    PGV['distance'] = [distance.distance(location, x).km for x in coordinates]
    PGV_fit = PGV.copy()
    # compare PGV['time'] to SIL origin time
    PGV_fit['time'] = [PGV_time + timedelta(0, x) for x in PGV_fit['time']]
    i = 0
    j = 12
    fit = 0.68
    for index, row in PGV_fit.iterrows():
        if 64.55 <= location[0] <= 64.7 and -17.65 <= location[1] <= -17.1:
            j = 25
        t1 = SIL_time + timedelta(0, row.distance/7.3 - 2)
        t2 = SIL_time + timedelta(0, row.distance/1.9 + j)
        check = t1 < row.time < t2
        if check:
            i += 1

    if len(PGV_fit.index) != 0 and i >= 10 and i/len(PGV_fit.index) >= fit:
        PGV_fit['fit'] = '{}/{} ({:.0f}%) '.format(i, len(PGV_fit.index),
                                                   (i/len(PGV_fit.index))*100)
        print(str(i) + '/' + str(len(PGV_fit.index)), end=' -> ')
        return True, PGV_fit
    else:
        print(str(i) + '/' + str(len(PGV_fit.index)), end=' -> ')
        print('not enough PGV within limits', end=' -> ')
        return False, PGV_fit


def find_alertevent(event_id, sil_path="/net/granit/mnt/sdc1/sk2/",
                    pre_path="/net/hellir/mnt/sdc1/alert'",
                    eps=150, time=None, location=[], logger=__name__):
    """
    Find alerts that fit evb, check their fit.

    Additionally checks the event file pgv times against distance to origin if
    location, origin time and station list with coodinates are provided.

    Parameters
    ----------
    event_id : str
        SIL event id = path to evb|eve|ana files.
        Example: "/2020/jun/25/00:05:00/09:33:883"
    sil_path : str, optional
        Path to SIL top dir which contains the year dirs. The default is
        "/net/granit/mnt/sdc1/sk2/"
    pre_path : str, optional
        Path to local alertMap output dirs. The default is
        "/net/hellir/mnt/sdc1/alert"
    eps : int, optional
        Time tolerance in seconds for associating events from SIL and
        alertMap. The default is 30.
    time: datetime
        time of origin gotten from cleanaut.mag
        if not provided will result in no check of PGV times.
    location: list, optional
        should be a list of latitude and longitude of origin from evb file.
        if not provided will result in no check of PGV times.
    logger : str, optional
        Name of logger object. The default is __name__ (current method name).

    Returns
    -------
    PGV : pandas.DataFrame
        Has index = station code, columns = lon, lat, time, pgv, fb, distance,
        fit, event; for stations in AlertMap *.event file. if returned from
        check_event_fit otherwise is an empty dataframe.
    """
    import os
    import csv
    import glob

    from datetime import datetime as dt
    from datetime import timedelta
    import pandas as pd

    # read EVB file
    filename = os.path.join(sil_path, event_id + ".evb")
    EVB = read_evb(filename, logger="__name__")

    # read time in header of all event files from the day
    tmp_path = os.path.join(pre_path, event_id[:12])
    # (event_id[:4] + '.old') til að komast í .old möppur

    files = glob.glob(tmp_path+'*.event')
    fields = {}
    for f in files:
        with open(f, 'r') as infile:
            reader = csv.DictReader(infile)
            fields[reader.fieldnames[0].split()[0]] = reader.fieldnames[0].split()[5:9]

    # set time window to look fo events within
    eps = timedelta(seconds=eps)
    window_min = (dt.combine(dt.today(), EVB.time.min()) - eps).time()
    window_max = (dt.combine(dt.today(), EVB.time.max()) + eps).time()

    # search if the event fits into the time
    # if location and time are given check the PGV time fits  time from origin.
    for key, value in fields.items():
        t = '{}/{}/{} {}'.format(value[0], value[1], value[3], value[2])
        PGV_time = dt.strptime(t, "%b/%d/%Y %H:%M:%S")
        event_found = window_min < PGV_time.time() < window_max
        if event_found:
            file = os.path.join(tmp_path, key + ".event")
            PGV = read_alert(file)
            if location and time is not None:
                check, PGV_checked = check_event_fit(location, time, PGV_time,
                                                     PGV)
                if check:
                    print('event found for {} - event file {}'.format(event_id,
                                                                      key))
                    PGV_checked['event'] = key
                    return PGV_checked
            else:
                return PGV  # without location and time takes first event found

    print('no event found for {}'.format(event_id))
    return pd.DataFrame()  # return empty dataframe if no event found


def main():
    import logging
    import logging.config

    log_dict = { 
         'version': 1, 
         'disable_existing_loggers': False, 
         'formatters': { 
             'standard': { 
                 'format': '%(asctime)-15s [%(levelname)s] %(funcName)s: %(message)s' 
             }, 
         }, 
         'handlers': { 
             'default': { 
                 'level': 'WARNING', 
                 'formatter': 'standard', 
                 'class': 'logging.StreamHandler', 
             }, 
             'file_handler': { 
                 'level': 'INFO', 
                 'filename': '/tmp/mylogfile.log', 
                 'class': 'logging.FileHandler', 
                 'formatter': 'standard' 
             } 
         }, 
         'loggers': { 
             'default': { 
                 'handlers': ['default'], 
                 'level': 'WARNING', 
                 'propagate': True 
             }, 
             'debug': { 
                 'handlers': ['default'], 
                 'level': "DEBUG", 
                 'propagate': True  
      
             }, 
             'info': { 
                 'handlers': ['default'], 
                 'level': "INFO", 
                 'propagate': True  
      
             }, 
             'geo_dataread.sil_read': {  
             'handlers': ['default'],  
             'level': "WARNING",  
             'propagate': True  
             }, 
              
         } 
     }                  
    
    logging.config.dictConfig(log_dict)

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

def readme():
    with open('README.rst') as f:
        return f.read()

setup(
      name='dblib',
      version='1.0',
      packages = find_packages(),
      description='Database library for use in GPS management',
      url='http://repo.vedur.is:18080/svn/GPS/library/dblib/branches/1.0',
      author='Fjalar Sigurdarson',
      author_email='fjalar@vedur.is',
      )



################################
#
# connect.py 
#
# Code made by fjalar@vedur.is 
# Iceland Met Office
# 2011-2012
#
# ###############################


######## IMPORT LIBRARIES ########

#import sys, re, subprocess, getopt, pyinotify, socket, os, serial
import sys, subprocess
#from time import sleep, time


class Connect:

    # ALL TEXT HERE IN THE DOC NEEDS TO BE REVISED LATER ON
    """A class designed to simplify communications between central server and equipment over different protocols and located behinds long links of connections.
    
    Public functions:
    open                -- 
    close               -- 
    getStatus           -- 


    Private functions:
    __openSSH           -- 
    __closeSSH          --
    __checkSSH          --
    __openSerial        --
    __closeSerial       --
    __checkSerial       --
    __openPOT           --
    __closePOT          --
    """

    def __init__(self,stationInfo):
        
        # 1 # Parameters saved and type set to string
        self.DEBUG = False
        self.TUNNEL_UP = 'Tunnel is up'
        self.TUNNEL_DOWN = 'Tunnel is down'

        # 2 # The info parameter is a dictionary that contains five other dictionaries as seen here below, 
        # provided by the configParser from the station config file:
        self.station    = stationInfo['station']    # id, name   
        self.receiver   = stationInfo['receiver']   # type, ip, httpport, ftpport, telnetport, serialport, 
                                                    # ...rtpport, reftekport, user, pwd, serv, baud, parity, rtscts
        self.connection = stationInfo['connection']    # type, ppp, port
        self.proxy      = stationInfo['proxy']        # user, server, host, httpport, ftpport, serialport
        self.router     = stationInfo['router']     # type, ip, phonenumber, portforward
        self.dialup     = stationInfo['dialup']     # device, phoneneumber, meth, server
        self.url        = 'locahost:%s' % self.proxy['httpport'] # self.url is returned by Connect() as the address for the port-forward.

    # ------------------ connect() ------------------ #

    def connect(self, reset=False):
        """This function opens a new connection for local tunnel and Proxy tunnel if they are down. If RESET is required, tunnels that already are up are shut down and new created."""

        # 1 # Name of the current module
        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print '%s connect() is now running... ' % currFunc

        # 2 # Define variables
        result = {'url':'','status':'','comm':''}

        # 3 # Establish connections
        # 3.1 # CONNECTION TYPE: IP direct connections
        if 'IP' in self.connection['type']  and 'direct' in self.connection['type']:

            # 3.1.1 Define variables        
            tunnel = 'direct'

            # 3.1.2 Mix results and done!
            result['url'] = '%s:%s' % (self.router['ip'],self.receiver['httpport'])
            result['status'] = 'ready'
            result['comm'] = 'Direct connection. No tunnels'

        # 3.2 # CONNECTION TYPE: IP via host connections             
        if 'IP' in self.connection['type']  and 'host' in self.connection['type'] and not 'sshserver' in self.connection['type']:

            # 3.2.1 Define variables
            #tunnel ={'local':'','comm':''}

           # 3.3.2 # Check current SSH status
            tunnel = self.__checkSSHTunnel('local')   # Check connection status

            # 3.2.3 If reset required and tunnel is up... take it down..
            if reset: 
                if self.DEBUG: print '%s Reset in progress...' % currFunc
                if tunnel['status'] == self.TUNNEL_UP:
                    tunnel = self.__closeSSHTunnel('local')  # If up; close localhost tunnel
                    if self.DEBUG: print '%s Localhost tunnel status: %s' % (currFunc, tunnel['status'])

            # 3.2.4 # Check the local tunnel and open if down
            if tunnel['status'] == self.TUNNEL_DOWN:  
                if self.DEBUG: print '%s Local tunnel is down. Opening one now...' % currFunc            
                tunnel = self.__openSSHTunnel('local')    # If everything else fails, open connection.
                if self.DEBUG: print '%s Local tunnel status: %s' % (currFunc, tunnel['status'])

            # 3.2.5 Mix results and done!!
            result['url'] = 'localhost:%s' % self.proxy['httpport']
            if tunnel['status'] == 'Tunnel is up':
                result['status'] = 'ready'
            else:
                result['status'] = 'not ready'
            
            result['comm'] = tunnel 


        # 3.3 # CONNECTION TYPE: IP via ssh_server via host connections             
        if 'IP' in self.connection['type']  and 'host' in self.connection['type'] and 'sshserver' in self.connection['type']:

            # 3.3.1 Define variables
            tunnel ={'local':'','local_comm':'','proxy':'','proxy_comm':''}
            
            # 3.3.2 # Check current SSH status
            tunnel_proxy = self.__checkSSHTunnel('proxy')   # Check connection status
            tunnel_local = self.__checkSSHTunnel('local')   # Check connection status

           # 3.3.3 # If reset required and tunnel is up... take it down..
            if reset: 
                if self.DEBUG: print '%s Reset in progress...' % currFunc
                if tunnel_local['status'] == self.TUNNEL_UP:
                    tunnel_local = self.__closeSSHTunnel('local')  # If up; close localhost tunnel
                    if self.DEBUG: print '%s Localhost tunnel status: %s' % (currFunc, tunnel_local['status'])
                if tunnel_proxy['status'] == self.TUNNEL_UP:
                    tunnel_proxy = self.__closeSSHTunnel('proxy')    # If up; close Proxy tunnel
                    if self.DEBUG: print '%s Proxy tunnel status: %s' % (currFunc, tunnel_proxy['status'])

            # 3.3.3 # Check the local tunnel and open if down
            if tunnel_local['status'] == self.TUNNEL_DOWN:  
                if self.DEBUG: print '%s Local tunnel is down. Opening one now...' % currFunc            
                tunnel_local = self.__openSSHTunnel('local')    # If everything else fails, open connection.
                if self.DEBUG: print '%s Local tunnel status: %s' % (currFunc, tunnel_local['status'])

            # 3.3.4 # Check the Proxy tunnel and open if down
            if tunnel_proxy['status'] == self.TUNNEL_DOWN:  
                if self.DEBUG: print '%s Proxy tunnel is down. Opening one now...' % currFunc            
                tunnel_proxy = self.__openSSHTunnel('proxy')    # If everything else fails, open connection.
                if self.DEBUG: print '%s Proxy tunnel status: %s' % (currFunc, tunnel_proxy['status'])

            # 3.3.6 check the status once more and return the results
            tunnel_proxy = self.__checkSSHTunnel('proxy')
            tunnel['proxy'],tunnel['proxy_comm'] = tunnel_proxy['status'],tunnel_proxy['comm']
            tunnel_local = self.__checkSSHTunnel('local')
            tunnel['local'],tunnel['local_comm'] = tunnel_local['status'],tunnel_local['comm']

            # 3.3.7 Mix results and done!!
            result['url'] = 'localhost:%s' % self.proxy['httpport']
            if tunnel['local'] == 'Tunnel is up' and tunnel['proxy'] == 'Tunnel is up':
                result['status'] = 'ready'
            else:
                result['status'] = 'not ready'
            
            result['comm'] = tunnel 

        # 3.4 # CONNECTION TYPE: SERIAL via host connections 
        if 'SERIAL' in self.connection['type']  and 'host' in self.connection['type']:
            print 'IP and host'


        # 3.5 # CONNECTION TYPE: SERIAL via ssh_server via host connections 
        if 'SERIAL' in self.connection['type']  and 'host' in self.connection['type'] and 'sshserver' in self.connection['type']:
            print 'IP, host and sshserver'


        # 4 # Return        
        return result

    # ------------------ disconnect() ------------------ #

    def disconnect(self):
        # Name of the current module
        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print '%s disconnect() is now running...' % currFunc
 
        result = {'status':'','comm':''}

        # 3.1 # CONNECTION TYPE: IP direct connections
        if 'IP' in self.connection['type']  and 'direct' in self.connection['type']:
            result = {'status':'Nothing to disconnect'}

        # 3.2 # CONNECTION TYPE: IP via host connections             
        if 'IP' in self.connection['type']  and 'host' in self.connection['type'] and not 'sshserver' in self.connection['type']:

            # 3.2.1 # Request the tunnels be closed
            tunnel = self.__closeSSHTunnel('local')

            # 3.2.2 # Manage the limited results accordingly and mix final result
            if tunnel['status'] == 'Tunnel is down':
                result['status'] = tunnel['status']
                result['comm'] = tunnel['comm']
                 
            if tunnel['status'] != 'Tunnel is down':
                result['status'] = 'Local tunnel is still up'
                result['comm'] = tunnel['comm']

        # 3.3 # CONNECTION TYPE: IP via ssh_server via host connections             
        if 'IP' in self.connection['type']  and 'host' in self.connection['type'] and 'sshserver' in self.connection['type']:
            
            # 3.3.1 # Request the tunnels be closed
            tunnel_proxy = self.__closeSSHTunnel('proxy')
            tunnel_local = self.__closeSSHTunnel('local')

            # 3.3.2 # Manage the limited results accordingly and mix final result
            if tunnel_proxy['status'] == 'Tunnel is down' and tunnel_local['status'] == 'Tunnel is down':
                result['status'] = 'All tunnels are down'
                result['comm'] = ''
                 
            if tunnel_proxy['status'] == 'Tunnel is down' and tunnel_local['status'] != 'Tunnel is down':
                result['status'] = 'Local tunnel is still up'
                result['comm'] = tunnel_local['comm']

            if tunnel_proxy['status'] != 'Tunnel is down' and tunnel_local['status'] == 'Tunnel is down':
                result['status'] = 'Proxy tunnel is still up'
                result['comm'] = tunnel_proxy['comm']

            if tunnel_proxy['status'] != 'Tunnel is down' and tunnel_local['status'] != 'Tunnel is down':
                result['status'] = 'Both tunnels are still up'
                result['comm'] = (tunnel_proxy['comm'],tunnel_local['comm'])

        # 3.4 # CONNECTION TYPE: SERIAL via host connections 
        if 'SERIAL' in self.connection['type']  and 'host' in self.connection['type']:
            print 'IP and host'

        # 3.5 # CONNECTION TYPE: SERIAL via ssh_server via host connections 
        if 'SERIAL' in self.connection['type']  and 'host' in self.connection['type'] and 'sshserver' in self.connection['type']:
            print 'IP, host and sshserver'


        return result
    # ------------------ checkStatus() ------------------ #

    def getStatus(self):
        # Name of the current module
        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print '%s getStatus() is now running...' % currFunc
        
        result = {'status':'','comm':''}

        # 3.1 # CONNECTION TYPE: IP direct connections
        if 'IP' in self.connection['type']  and 'direct' in self.connection['type']:
            result = {'status':'Direct connection. No tunnels.'}

        # 3.2 # CONNECTION TYPE: IP via host connections             
        if 'IP' in self.connection['type']  and 'host' in self.connection['type'] and not 'sshserver' in self.connection['type']:

            result = self.__checkSSHTunnel('local')

            if self.DEBUG: 
                if result['status'] != '': print 'Tunnel on localhost: %s' % result['status']
                if result['comm'] != '': print 'Message on localhost: %s' % result['comm']


        # 3.3 # CONNECTION TYPE: IP via ssh_server via host connections             
        if 'IP' in self.connection['type']  and 'host' in self.connection['type'] and 'sshserver' in self.connection['type']:

            result = {'local':'','local_comm':'','proxy':'','proxy_comm':''}
            tunnel_proxy = self.__checkSSHTunnel('proxy')
            tunnel_local = self.__checkSSHTunnel('local')

            result['proxy'],result['proxy_comm'] = tunnel_proxy['status'],tunnel_proxy['comm']
            result['local'],result['local_comm'] = tunnel_local['status'],tunnel_local['comm']

            if self.DEBUG: 
                if result['local'] != '': print 'Tunnel on localhost: %s' % result['local']
                if result['local_comm'] != '': print 'Message on localhost: %s' % result['local_comm']
                if result['proxy'] != '': print 'Tunnel on Proxy server: %s' % result['proxy']
                if result['proxy_comm'] != '': print 'Message on Proxy server: %s' % result['proxy_comm']

        # 3.4 # CONNECTION TYPE: SERIAL via host connections 
        if 'SERIAL' in self.connection['type']  and 'host' in self.connection['type']:
            print 'IP and host'

        # 3.5 # CONNECTION TYPE: SERIAL via ssh_server via host connections 
        if 'SERIAL' in self.connection['type']  and 'host' in self.connection['type'] and 'sshserver' in self.connection['type']:
            print 'IP, host and sshserver'

        
        return result

    # ------------------ setDebug() ------------------ #

    def setDebug(self):
        """Debug ON/OFF switch."""
        if self.DEBUG:
            self.DEBUG = False
            print 'Debug OFF'
        else:
            self.DEBUG = True
            print 'Debug ON'    
    
    # ------------------ info ------------------ #

    def info(self):
        print 'Station ID: ', self.station['id']
        print 'Connection type: ', self.connection['type']
        if 'proxy' in self.connection['type']:
            print 'Proxy server: ',self.proxy['sshserver']
            print 'Proxy host: ',self.proxy['host']
        else:
            print 'Router IP: ', self.router['ip']
        print '....'


    #------------------- __openSSHTunnel ------------------- #

    def __openSSHTunnel(self,location=''):

        # 1 # Define variables
        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'    # Name of the current module 
        if self.DEBUG: print '%s _openSSHTunnel() is now running for location >> %s <<' % (currFunc, location) 

        result = {'status':'','comm':''}        

        # For "few" (one) station the ssh port is defined differently from the standard 22
        sshport = ''
        if 'sshserver' in self.connection['type']: 
            if location == 'proxy' and self.connection['sshport'] != '':
                if self.DEBUG: print ' %s Different ssh port defined for proxy connection: %s ' % (currFunc, self.connection['sshport'])
                sshport = '-p %s' % self.connection['sshport']
        elif 'host' in self.connection['type'] and not 'sshserver' in self.connection['type']:
            if location == 'local' and self.connection['sshport'] != '':
                if self.DEBUG: print '%s Different ssh port defined for local connection: %s ' % (currFunc, self.connection['sshport'])
                sshport = '-p %s' % self.connection['sshport']

        # Do we need a local tunnel?        
        if location == 'local':

            if 'sshserver' not in self.connection['type']:
                command = "ssh %s -fqN -L %s:%s:%s %s@%s" % (sshport, self.proxy['httpport'],self.receiver['ip'],self.receiver['httpport'],self.proxy['user'],self.proxy['host'])
            else:
            # SSH shell command to open a ssh tunnel between host <-> proxy <-> destionation
                command = "ssh %s -fqN -L %s:localhost:%s %s@%s" % (sshport, self.proxy['httpport'],self.proxy['httpport'],self.proxy['user'],self.proxy['sshserver'])        
            if self.DEBUG: print '%s Command to open local tunnel: %s' % (currFunc, command)

            # WARNING:
            #   process.communicate(), the stout communication, can not be saved to a variable here. 
            #   The ssh tunnel goes into background but the python process will hang and wait for the stout
            #   which never comes back...

            # 2 # Execute the command
            return_code, return_comm = self.__shellComm(command, comm='no') # Execute shell command and return code. Communication cannot be saved here. 
            
            # 3 # Manage the output accordingly
            if self.DEBUG: print '%s process.returncode: %s' % (currFunc , return_code)

            if return_code == 0:
                result['status'],result['comm'] = (self.TUNNEL_UP,return_comm)
                if self.DEBUG: print '%s Tunnel is up!' % currFunc
            elif return_code == 255:
                result['status'],result['comm'] = ('Connection timed out...',return_comm)
                if self.DEBUG: print '%s Connection timed out. Return code: %s' % (currFunc , return_code) 
            else:
                result['status'], result['comm'] = ('Tunnel failed to open.', return_comm)
                if self.DEBUG: print return_code

        # Do we need a local tunnel?
        if location == 'proxy':    
            # 4.3.1 # Define variables

            command = "ssh -t %s@%s 'ssh %s -fqN -L %s:%s:%s %s@%s'" % (   self.proxy['user'],self.proxy['sshserver'], sshport, self.proxy['httpport'],self.receiver['ip'],
                                                                        self.receiver['httpport'],self.proxy['user'],self.proxy['host'])
            
            if self.DEBUG: print '%s Command to open Proxy tunnel: %s' % (currFunc , command)

            # 4.3.2 # Execute command

            # WARNING:
            #   process.communicate(), the stout communication, can not be saved to a variable here. 
            #   The ssh tunnel goes into background but the python process will hang and wait for the stout
            #   which never comes back...

            return_code, return_comm = self.__shellComm(command,comm='no')
            if self.DEBUG: print '%s process.returncode: %s ' % (currFunc, return_code)

            if return_code == 0:
                result['status'],result['comm'] = (self.TUNNEL_UP,return_comm)
                if self.DEBUG: print '%s Tunnel is up!' % currFunc
            elif return_code == 255:
                result['status'],result['comm'] = ('Connection timed out...',return_comm)
                if self.DEBUG: print '%s Connection timed out. Return code: %s' % (currFunc , return_code) 
            else:
                result['status'], result['comm'] = ('Tunnel failed to open.', return_comm)
                if self.DEBUG: print return_code

        # 4 # Return returncode and port number
        return result

    #------------------- __checkSSHTunnel------------------- #

    def __checkSSHTunnel(self,location=''):
        """ This function checks if tunnel is up and active on the localhost and on a Proxy server, if needed."""

        # 1 # Define variables 
        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name
        result = {'status':'','comm':''}

        if self.DEBUG: print '%s _checkSSHTunnel() is now running for location >> %s <<' % (currFunc, location)
        
        if location == 'local': # 2 # Check local tunnel

            # 2.1 # Define a 'find tunnel' command with ps... look into suggestion from olstar.
            command = "ps -ef | grep 'ssh' | grep %s | grep -v root | awk '{print $2}'" % self.proxy['httpport']
            if self.DEBUG: print '%s Command to find process on localhost: %s' % (currFunc, command)

            # 2.2 # Execute command
            return_code, return_comm = self.__shellComm(command,comm='yes')        

            # 2.3 # Process return code and communication
            if return_code == 0:
                if return_comm != '': #if the result contains some information...
                    if self.DEBUG: print '%s Open tunnel found on localhost with process number: %s' % (currFunc, return_comm)
                    result['status'],result['comm'] = (self.TUNNEL_UP, return_comm) # Save results to result dictionary
                else:
                    if self.DEBUG: print '%s No tunnel found on localhost...' % (currFunc)
                    result['status'],result['comm'] = (self.TUNNEL_DOWN,return_comm) # Save results to result dictionary
            else:
                print '%s Error in running command | Returncode: %s | Return comm: %s'% (currFunc, return_code, return_comm)

        if location == 'proxy': # 3 # Check Proxy tunnel

            # 3.1 # Define a 'find tunnel' command with ps... look into suggestion from olstar.
            command = "ssh -t %s@%s 'sh /usr/sil/gps/bin/find_tunnel.sh %s %s'" % (self.proxy['user'],self.proxy['sshserver'],self.proxy['httpport'],self.receiver['ip'])
            if self.DEBUG: print '%s Command to find process on Proxy server: %s' % (currFunc,command)

            # 6.2 # Execute the command.
            return_code, return_comm = self.__shellComm(command,comm='yes')        
            
            # 6.3 # Process return code and communication
            if return_code == 0:
                if return_comm != '': #if the result contains some information...
                    if self.DEBUG: print '%s Open tunnel found on Proxy server with process number: %s' % (currFunc, return_comm)
                    result['status'],result['comm'] = (self.TUNNEL_UP, return_comm) # Save results to result dictionary
                else:
                    if self.DEBUG: print '%s No tunnel found on Proxy server...' % (currFunc)
                    result['status'],result['comm'] = (self.TUNNEL_DOWN,return_comm) # Save results to result dictionary
            else:
                print '%s Error in running command | Returncode: %s | Return comm: %s'% (currFunc, return_code, return_comm)

        return result
    
    #------------------- __closeSSHTunnel ------------------- #

    def __closeSSHTunnel(self,location=''):
        """ This function closes ssh tunnels on localhost and Proxy server """

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name

        result = {'status':'','comm':''}

        if self.DEBUG: print '%s __closeSSHTunnel() is now running for location >> %s <<' % (currFunc, location)

        if location == 'proxy':

            result = self.__checkSSHTunnel('proxy')

            if result['status'] == self.TUNNEL_UP:
        
                #(3a)# Define 'Close tunnel' command
                if self.DEBUG: print "%s Tunnel process found on Proxy server and will be closed now." % currFunc
                command = "ssh -t %s@%s 'kill %s'" % (self.proxy['user'],self.proxy['sshserver'],result['comm'])
                if self.DEBUG: print '%s Command to close tunnel on Proxy server: %s' % (currFunc,command)

                return_code, return_comm = self.__shellComm(command,comm='yes')

                if return_code == 0:
                    if self.DEBUG: print '%s Tunnel on Proxy server has been closed.' % currFunc
                    result['status'],result['comm'] = self.TUNNEL_DOWN, return_comm
                else:
                    if self.DEBUG: print '%s Problem closing tunnel on Proxy server | Return code: %s.' % (currFunc, return_code)
                    result['status'],result['comm'] = self.TUNNEL_DOWN, 'Problem closing tunnel on Proxy server: %s' % return_code
  

        if location == 'local':

            result = self.__checkSSHTunnel('local')

            if result['status'] == self.TUNNEL_UP:
        
                #(3a)# Define 'Close tunnel' command
                if self.DEBUG: print "%s Tunnel process found on localhost and will be closed now." % currFunc
                command = "kill %s" % result['comm']

                if self.DEBUG: print '%s Command to close tunnel on localhost: %s' % (currFunc,command)

                return_code, return_comm = self.__shellComm(command,comm='yes')

                if return_code == 0:
                    if self.DEBUG: print '%s Tunnel on localhost has been closed.' % currFunc
                    result['status'],result['comm'] = self.TUNNEL_DOWN, return_comm
                else:
                    if self.DEBUG: print '%s Problem closing tunnel on localhost | Return code: %s.' % (currFunc, return_code)
                    result['local'],result['comm'] = self.TUNNEL_DOWN,'Problem closing tunnel on localhost: %s' % return_code

        return result
        
 
    #------------------- __shellComm ------------------- #

    def __shellComm(self,command,comm='yes'):
        """ This command executes shell commands and returns the results"""

        return_code = ''
        return_comm = ''

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'

        if self.DEBUG: print '%s __shellComm() is now running...' % currFunc

        process = subprocess.Popen( command , shell=True , stdout=subprocess.PIPE) 
        process.wait()
        return_code = process.returncode

        if comm == 'yes':
            return_comm = process.communicate()[0].strip('\n').strip('\r') # strip out the extra hidden characters (newline and carrage return)

        if self.DEBUG: print '%s process.returncode: %s ' % (currFunc, return_code)

        return return_code, return_comm       




#!/usr/bin/python
# -*- coding: utf-8 -*-

# ###############################
#
# proxytunnel.py
# Code made by fjalar@vedur.is
# Iceland Met Office
# 2013-2014
#
# ###############################


import cparser as parser, pconnect as connect
import sys, getopt,re

def run(p_args):

    # 1 # instatniate a parser object with a specific station
    Parser = parser.Parser()
    #print "Check #1"

    # 2 # get station info for that station
    if p_args['debug']: print "Station: %s" % p_args['sid']
    info = Parser.getStationInfo(p_args['sid'])
    if p_args['debug']: print "Check #2"
    if p_args['debug']: print "INFO: ",info

    # 3 Check if everything is ok
    if info: 

        # 4 # instantiate a connection object for a specific station
        Connection = connect.Connect(info)
        #print "Check #3"

        # 5 # enable debug
        if p_args['debug']: Connection.setDebug()


        if p_args['action'] == 'open':

	    print "Action: Open connection for %s" % p_args['sid']

	    # 6a # Try disconnect existing connection first - this is done twice just to be sure 
	    status_dis_1 = Connection.disconnect()
	    status_dis_2 = Connection.disconnect()

            # 7a # finally connect and catch the status output in the status variable
            status_con = Connection.connect()

	    # 8a # report if not silent
            if p_args['report'] != 'norep': 
	        print status_dis_1
	        print status_dis_2
	        print status_con


        elif p_args['action'] == 'close':

            print "Action: Close connection for %s" % p_args['sid']

            # 6b # Disconnect existing connection
            status_dis_1 = Connection.disconnect()
            status_dis_2 = Connection.disconnect()

            # 7b # report if not silent
            if p_args['report'] != 'norep':
                print status_dis_1
                print status_dis_2

    else:
       print "ERROR: Problem with config file"

######## function - HELP SCREEN #######

def helpScreen():

    # Only help info here
    print "ProxyTunnel is a tunneling through proxy tool that opens a tunnel connection through a proxy to an GPS equipment"
    print ""
    print "Usage:"
    print "  proxytunnel [OPTIONS] [ARGUMENS]"
    print ""
    print "Examples:"
    print "  (1) proxytunnel -o GOLA   - Opens a connection to the GPS station GOLA through proxies kjarni and god"
    print "  (2) proxytunnel -c FTEY   - Closes a connection to the GPS station FTEY through proxies kjarni and fla"
    print ""
    print "Opitions:"
    print "  -o, --open=[station id]       Open a connection."
    print "  -c, --close=[station id]      Close a open/live connection."
    print "  -v, --verbose                 Verbose reporting."
    print "  -n, --no-report               No reporting - Quiet mode."
    print "  -d, --debug                   Print out debug information."
    print ""
    print "Default reporting level is modest."
    print ""


######## function - PROGRAM INFO SCREEN #######

def progInfoScreen():

    # Only splash screen info here
    print ""
    print "Copyright (c) 2013-2014 Icelandic Met Office"
    print "ProxyTunnel 0.1 (21st of November 2013)"
    #print ""

######## function - PROGRAM ERROR OUTPUT SCREEN #######

def errorScreen(message):

    helpScreen()
    print ""
    print "ERROR: %s" % message


def main():


    currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
    progInfoScreen()
    sid = []

    # Variables defined

    p_args = {'sid':None,'report':'default','debug':None}
    # 'report' can be norep, default or verbose. Default is 'default'
    # 'file' is station-list.cfg as default and the file is located under ../../cofigs/ directory.

    # Option and argument parser.

    try:
        opts, args = getopt.getopt(sys.argv[1:],"dhocvn", ["debug","help","open","close","verbose","no-reporting"])
        # semicolon following a option stands for an argument
    except getopt.error, msg:
        helpScreen() #Print out the help screen
        print "ERROR: ", msg
        sys.exit(2)

    # DEBUG
    #print len(sys.argv[1:])
    #print sys.argv[1:]
    #for option, argument in opts:
        #print "option: ",option
        #print "argument: ",argument
    #sys.exit(2)
    # DEBUG

    # If there are no arguments or options print out the help screen.
    if len(sys.argv[1:]) == 0:
        helpScreen()

    # Else, process the options and arguments.
    else:

        for option, argument in opts:
            if option in ("-d","--debug"):
                #print "option: %s " % option
                 p_args['debug'] = 1
                 print "Option(s): %s " % option

            elif option in ("-h", "--help"):
                helpScreen()
                #print "option: %s  a: %s" % (option,argument)
                sys.exit(0)

            elif option in ("-o","--open"):
                p_args['action'] = 'open'
                #p_args['sid'] = argument
                #print "option: %s " % option

            elif option in ("-c","--close"):
                p_args['action'] = 'close'
                #p_args['sid'] = argument
                #print "option: %s " % option

            elif option in ("-v","--verbose"):
                print "option: %s " % option
                #p_args['report'] = 'verbose'

            elif option in ("-n","--no-reporting"):
                print "option: %s" % option

            else:
                assert False, "unhandled option"

        if p_args['debug']: print 'Arguments: ', args
        try:
            if p_args['debug']: print "%s in the try-except method" % currFunc

            matchObj = re.search( r'[A-Z][A-Z][A-Z][A-Z0-9]', args[0], flags=0)
            if matchObj:
                sid.append(matchObj.group())
            
                if p_args['debug']: print "%s >> %s will be processed..." % (currFunc,matchObj.group())

                p_args['sid'] = matchObj.group()
                run(p_args)

        except:
            errorScreen('Station ID missing. Use ALL to process all stations or give a specific station ID, e.g. SOHO')
            sys.exit(0)



if __name__=="__main__":
     main()



from setuptools import setup, find_packages

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='cparser',
      version='1.0',
      description='Config parser for GPS Station files',
      url='http://repo.vedur.is:18080/svn/GPS/library/cparser/branches/1.0',
      author='Fjalar Sigurdarson',
      author_email='fjalar@vedur.is',
      license='Icelandic Met Office',
      packages=find_packages(),
      include_package_data=True,
      )



# -*- coding: utf-8 -*-
from __future__ import print_function
"""
Reading in gas data from IMO

functions ror read in data: 
def read_gas_data(station, device,  start=None, end=None, dsource="FILE", wrfile=True, 
                  fname=None, with_excluded=True, id_observation_start=False, conn_dict=False, 
                  logging_level=logging.INFO):

functions for returning subsets:
"""

# used in  a function arguments
#import geofunc.geo as geo 
import logging

#
# Initital trivial filtering
#


def mask_first(x):

    """
    """
    import numpy as np

    x[0] = False
    
    return x

def init_multigas(data):
    """
    """
    import pandas as pd  
    pd.set_option('mode.chained_assignment',None) 
    

    data['filter']=True
    mask = data.groupby(['acquisition'])['filter'].transform(mask_first)
    data = data.loc[mask]
    #data.loc[:,'co2_concentration'] *= 0.0001 # data['co2'].multiply(0.0001)

    if "co2_concentration" in data.columns:
        data['co2_concentration'] = data['co2_concentration'].apply( lambda x: x*0.0001)

    return data


#
# reading the gas data
#

def read_gas_data(station, device,  start=None, end=None, dsource="FILE", wrfile=True, 
                  fname=None, with_excluded=True, id_observation_start=False, conn_dict=False, 
                  logging_level=logging.INFO):
    """
    Reading in gas data and returning it as pandas.DataFrame
    """

    import requests
    import logging
    import psycopg2
    from psycopg2 import sql
    import pandas as pd
    import datetime as dt
    from datetime import timedelta as td
    import gtimes.timefunc as gt

    # String format for the rest service
    strf="%Y-%m-%d %H:%M:%S"

    # Handling logging
    logging.basicConfig(
        format="%(asctime)-15s [%(levelname)s] %(funcName)s: %(message)s",
        level=logging_level )

    logging.getLogger().setLevel(logging_level)
    module_logger = logging.getLogger()

    #pd.options.mode.chained_assignment = None

    if fname == None:
        #fname="gas/lagu.p"
        fname = "{0:s}-{1:s}.p.gz".format(station,device)
    
        

    data = pd.DataFrame()
    if dsource == "FILE":
        try: 
            module_logger.info("Reading gas data from file: {}".format(fname))
            data = pd.read_pickle(fname)

        except  IOError as e:
            module_logger.info("While reading File {}:  ".format(fname) + str(e))
        except  pd.errors.EmptyDataError as e:
            module_logger.info("While reading File {}:  ".format(fname) + str(e))
        except BaseException as e:
            module_logger.error("While reading File {}:  ".format(fname) + str(e))
            raise

        if start:
            data = data[(data.index > start)]
        if end:
            data = data[(data.index < end)]

    elif dsource == "DBASE":
        period = ""
        if start:
            period = period + "and o.acquisition >= '{}'".format(start.strftime(strf)) 
        if end:
            period = period + " and o.acquisition <= '{}'".format(end.strftime(strf)) 
        
        #if conn_dict:
        module_logger.info("Connecting to database:")
        module_logger.info("Connection info:\n{}".format(conn_dict))
        
        try:
            conn = psycopg2.connect( dbname=conn_dict["dbname"],
                                       user=conn_dict["user"],
                                   password=conn_dict["password"], 
                                       host=conn_dict["host"], 
                                       port=conn_dict["port"] )
        except TypeError as e:
            module_logger.warning("Your dicionary is not formated correctly it's value is:\n  {} \n".format(conn_dict) + str(e))
            raise
        except BaseException as e:
            module_logger.error("Error while connecting to database using {}\n:  ".format(conn_dict) + str(e))
            raise

        cursor = conn.cursor()

        sql_command="SELECT * FROM device;"

        query = ("select o.id, code, observation_time, acquisition, numeric_value \
                \
                 from observation o \
                 join device d \
                    on o.id_device= d.id \
                 join tos_station s \
                    on d.id_tos_station=s.id \
                 join parameter p \
                    on o.id_parameter=p.id \
                    \
                 where s.station_identifier = '{}' \
                 and  d.device_identifier = '{}'  {} \
                \
                order by o.observation_time").format(station,device,period)
        
        module_logger.info("Query info:\n" + str(query))

        #data.to_pickle("{}_{}.p.gz".format(station,device))
        
        try:
            data = pd.read_sql(query, conn)
            data = pd.concat([data.set_index('observation_time')['acquisition'],
                          data.pivot(  index='observation_time', columns='code', 
                          values='numeric_value')], axis=1).drop_duplicates()    
        except:
            pass

    elif dsource == "SERVICE":

        server="localhost"
        port=11111
        prepath="/aot/gas/v2/stations"

        url_rest="http://{}:{}{}/{}/devices/{}/observations".format(server,port,prepath,station,device)

        if end == None:
             end = dt.datetime.now()
        if start == None:
            start = gt.currDatetime(-30,refday=end) 

    
        #print(end-start)
        #read_blocks=gt.datepathlist(strf,'20D',start,end)
        #dspan = "&date_from={0:s}&date_to={1:s}".format(start.strftime(strf),end.strftime(strf))

        #dspan = "?date_to={0:s}".format(end.strftime(strf))
        #with_excluded="?with_excluded={0}".format(with_excluded)
    
        id_observation_start="&id_observation_start={0}".format(id_observation_start)
        param_string=with_excluded+id_observation_start+dspan

        #request = requests.get(url_rest+station_marker+'?date_from=2012-12-01 00:00:00&date_to=2017-12-31 00:00:00')
        #print(url_rest+param_string)
        request = requests.get(url_rest+param_string)
        data = pd.DataFrame.from_records(request.json()['data'],index='observation_time')
        data.index = pd.to_datetime(data.index)
        
    
        if device == "multigas":
            data = init_multigas(data)
        elif device == "crowcon":
            pass

    else:
        module_logger.error("Unregignised read method {}, use: \"FILE\", \"SERVICE\" or \"DBASE\" ".format(dsource))
        raise NameError("Unregignised source name {}".format(dsource))




    if wrfile == True:
        data.to_pickle(fname)
    try:
        module_logger.info("Dataframe columns:\n" + str(data.columns) + "\n")
        module_logger.info("Input time period: ({}, {})".format(start, end))
        module_logger.info("dataframe First and Last lines:\n"  + str(data.iloc[[0,-1]]))
        module_logger.debug("Dataframe shape: {}".format( str(data.shape) )) 
    except IndexError:
        module_logger.warning("Dataframe is empty {}".format( str(data) )) 


    #tbegin = dt.strptime(request.json()['temporalExtent']['begin'], "%Y-%m-%dT%H:%M:%S")
    #tend = dt.strptime(request.json()['temporalExtent']['end'], "%Y-%m-%dT%H:%M:%S")
    #print(tbegin)
    #print(tend)
    #print(tend-tbegin)
    return data




#!/usr/bin/python
"""
convllh(llh,radians=True)
extractfromGamitBakf(cfile, stations)
savedisp(dataDict,fname=None, header="")
"""

import numpy as np
import geofunc.geo as geo


def convllh(llh, radians=True):
    """
    input:
        llh: array_like longitude (deg/rad), latitude (deg/rad) and height (m)

    out:
        llh: numpy array longitude (rad/deg), latitude (rad/deg) and height (m)
    """

    if type(llh) is not np.ndarray:
        llh = np.array(llh)

    ll = llh.copy()

    if radians:
        ll[:, 0:2] = np.degrees(ll[:, 0:2])
    else:
        ll[:, 0:2] = np.radians(ll[:, 0:2])

    return ll


def plateVelo(locList, plate=None):
    """ """
    import pyproj as proj
    import cparser as cp
    import numpy as np

    staDict = {}

    NEU_vel = np.zeros([len(locList), 3], dtype="float64")

    f = open(cp.Parser().getPostprocessConfig()["coordfile"], "r")
    staDict.update(
        dict([[line.split()[3], list(map(float, line.split()[0:3]))] for line in f])
    )

    index = 0
    for sta in locList:
        if plate:
            rotp = geo.platePoles(plate)
        else:
            rotp = geo.platePoles(plateDict()[sta])
        if sta in staDict:
            disp = geo.rotpole(rotp, staDict[sta])
            llh = np.array(geo.itrf08towgs84.transform(*staDict[sta], radians=True)).transpose()
            llh[0:2] = np.deg2rad(llh[0:2])  # inplace of radians=True
            NEU_vel[index, :] = geo.eccell(llh, disp)
        else:
            NEU_vel[index, :] = np.zeros([1, 3], dtype="float64")
            print("WARNING: Cannot determine plate velocity at station {}".format(sta))
        index += 1

    return NEU_vel


def plateFullname(plate):
    """
    Maps tectonik plate short name to its full name
    (unfinished)
    """

    plateFullname = {
        "NOAM": "North American plate",
        "EURA": "Eurasian Plate",
        "NAZC": "Nazca",
        "INDI": "Indian Plate",
        "NUBI": "Nubian Plate",
        "ARAB": "Arabia",
    }

    return plateFullname[plate]


def plateDict():
    """
    returns station: plate shortname dictionary
    """
    import cparser as cp

    plateDict = {}

    f = open(cp.Parser().getPostprocessConfig()["pfile"], "r")
    for line in f:
        sta, plate = line.rstrip().split()
        plateDict[sta] = plate
    f.close()

    return plateDict


### routines to extract and save coordiantes and time series from gamit
def extractfromGamitBakf(cfile, stations):
    """ """
    import re

    slines = []

    site = re.compile(stations)
    tim = re.compile("Solution refers to", re.IGNORECASE)
    f = open(cfile, "r")

    for line in f:
        if site.search(line):  # or tim.search(line):
            slines.append(line.rstrip())

    return slines


def savedisp(dataDict, fname=None, header=""):
    """ """
    from collections import OrderedDict

    valtype = type(list(dataDict.values())[0])

    dataDict = OrderedDict(sorted(dataDict.items()))

    datavalues = list(dataDict.values())
    if (valtype is list) or (valtype is np.ndarray):
        fmt = "% 3.8f\t% 2.8f\t% 2.8f\t%s"
        ab = np.zeros(
            len(dataDict.keys()),
            dtype=[
                ("var1", "float"),
                ("var2", "float"),
                ("var3", "float"),
                ("var4", "a4"),
            ],
        )
        ab["var1"] = np.squeeze(datavalues)[:, 0]
        ab["var2"] = np.squeeze(datavalues)[:, 1]
        ab["var3"] = np.squeeze(datavalues)[:, 2]
        ab["var4"] = [str(item) for item in dataDict.keys()]
    if valtype is tuple:
        fmt = "% 3.8f\t% 2.8f\t% 2.8f\t%2.8f\t%2.8f\t%s"
        ab = np.zeros(
            len(dataDict.keys()),
            dtype=[
                ("var1", "float"),
                ("var2", "float"),
                ("var3", "float"),
                ("var4", "float"),
                ("var5", "float"),
                ("var6", "a4"),
            ],
        )
        ab["var1"] = np.squeeze(list(zip(*datavalues[:]))[0])[:, 0]
        ab["var2"] = np.squeeze(list(zip(*datavalues[:]))[0])[:, 1]
        ab["var3"] = np.squeeze(list(zip(*datavalues[:]))[1])[:, 0]
        ab["var4"] = np.squeeze(list(zip(*datavalues[:]))[1])[:, 1]
        ab["var5"] = np.squeeze(list(zip(*datavalues[:]))[1])[:, 2]
        ab["var6"] = [str(item) for item in dataDict.keys()]

    if fname:
        np.savetxt(fname, ab, fmt=fmt, header=header)
    return ab


def getStationCoordinates(station_list=None):
    """
    Return pandas dataframe with stations and longitude, latitude, height.
    INPUT: list of station codes (4ch names)
        if None, return all available stations
    OUTPUT: pandas dataframe (index:station IDs, colums: coordinates)
    EXAMPLE:
        station_list=['REYK','AUST','SVIN']
        sLLH = getStationCoordinates(station_list)
        sLLH
        [Out]
                    lon        lat           elev
        REYK -21.955488  64.138786      93.012961
        AUST -19.080568  63.674361    1438.359718
        SVIN -16.816561  64.009312     727.272716
    """
    import cparser as cp
    import geofunc.geo as geo
    import pandas as pd

    sta_llh = {}
    sta_locs = cp.Parser().getStationCoordinates()
    if station_list is None or not station_list:
        station_list = sta_locs.copy()
    for k in station_list:
        if k in sta_locs:
            sta_llh[k] = geo.xyzell(sta_locs[k], radians=False)  # lon,lat,elev.
        else:
            sta_llh[k] = np.array([None, None, None], float)
    df_sloc = pd.DataFrame.from_dict(sta_llh, orient="index")
    df_sloc.columns = ["lon", "lat", "elev"]
    return df_sloc



import pycurl, cStringIO, sys

class sCurl():

    # ALL TEXT HERE IN THE DOC NEEDS TO BE REVISED LATER ON
    """A simple CURL class to provide a simple interface on top of pycurl.
    Public functions:
    getURL     -- Get the url for the passed argument.
    info       -- Reports information on the object.
    """

    def __init__(self):
    
    # instantiate pycurl object
        self.curl = pycurl.Curl()


   
    def getURL(self,url,userpass, DEBUG):

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        
        #print 'getURL called witu URL sting: ',url_string        
        # set variables
        result = {'value':None,'error':None}
        response = ''
        error = ''

        # curl magic
        buf = cStringIO.StringIO()             
        
        try:
            self.curl.setopt(self.curl.URL, url)
	    self.curl.setopt(pycurl.CONNECTTIMEOUT, 4)
	    self.curl.setopt(pycurl.TIMEOUT, 4)
	    self.curl.setopt(pycurl.NOSIGNAL, 4)

            if userpass: self.curl.setopt(self.curl.USERPWD, userpass['user'] + ":" +  userpass['pass'])

            self.curl.setopt(self.curl.WRITEFUNCTION, buf.write)
            self.curl.perform()
            result['value'] = buf.getvalue()

            if DEBUG: print '%s Value retrieved: %s URL: %s' % (currFunc,result['value'],url)
        
        except pycurl.error, error:
            errno, errstr = error
            result['error'] = errstr
            if DEBUG: print '%s An error occurred: %s - URL: %s' % (currFunc,errstr,url)
        
        buf.close()

        return result
